# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from typing_extensions import Self

class BidOpportunity(BaseModel):
    """
    BidOpportunity
    """ # noqa: E501
    campaign_id: Optional[StrictStr] = Field(default=None, alias="CampaignId")
    ad_group_id: Optional[StrictStr] = Field(default=None, alias="AdGroupId")
    keyword_id: Optional[StrictStr] = Field(default=None, alias="KeywordId")
    current_bid: Optional[StrictFloat] = Field(default=None, alias="CurrentBid")
    match_type: Optional[StrictStr] = Field(default=None, alias="MatchType")
    suggested_bid: Optional[StrictFloat] = Field(default=None, alias="SuggestedBid")
    estimated_increase_in_impressions: Optional[StrictStr] = Field(default=None, alias="EstimatedIncreaseInImpressions")
    estimated_increase_in_clicks: Optional[StrictFloat] = Field(default=None, alias="EstimatedIncreaseInClicks")
    estimated_increase_in_cost: Optional[StrictFloat] = Field(default=None, alias="EstimatedIncreaseInCost")
    opportunity_key: Optional[StrictStr] = Field(default=None, alias="OpportunityKey")
    __properties: ClassVar[List[str]] = ["CampaignId", "AdGroupId", "KeywordId", "CurrentBid", "MatchType", "SuggestedBid", "EstimatedIncreaseInImpressions", "EstimatedIncreaseInClicks", "EstimatedIncreaseInCost", "OpportunityKey"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if campaign_id (nullable) is None
        # and model_fields_set contains the field
        if self.campaign_id is None and "campaign_id" in self.model_fields_set:
            _dict['CampaignId'] = None

        # set to None if ad_group_id (nullable) is None
        # and model_fields_set contains the field
        if self.ad_group_id is None and "ad_group_id" in self.model_fields_set:
            _dict['AdGroupId'] = None

        # set to None if keyword_id (nullable) is None
        # and model_fields_set contains the field
        if self.keyword_id is None and "keyword_id" in self.model_fields_set:
            _dict['KeywordId'] = None

        # set to None if current_bid (nullable) is None
        # and model_fields_set contains the field
        if self.current_bid is None and "current_bid" in self.model_fields_set:
            _dict['CurrentBid'] = None

        # set to None if match_type (nullable) is None
        # and model_fields_set contains the field
        if self.match_type is None and "match_type" in self.model_fields_set:
            _dict['MatchType'] = None

        # set to None if suggested_bid (nullable) is None
        # and model_fields_set contains the field
        if self.suggested_bid is None and "suggested_bid" in self.model_fields_set:
            _dict['SuggestedBid'] = None

        # set to None if estimated_increase_in_impressions (nullable) is None
        # and model_fields_set contains the field
        if self.estimated_increase_in_impressions is None and "estimated_increase_in_impressions" in self.model_fields_set:
            _dict['EstimatedIncreaseInImpressions'] = None

        # set to None if estimated_increase_in_clicks (nullable) is None
        # and model_fields_set contains the field
        if self.estimated_increase_in_clicks is None and "estimated_increase_in_clicks" in self.model_fields_set:
            _dict['EstimatedIncreaseInClicks'] = None

        # set to None if estimated_increase_in_cost (nullable) is None
        # and model_fields_set contains the field
        if self.estimated_increase_in_cost is None and "estimated_increase_in_cost" in self.model_fields_set:
            _dict['EstimatedIncreaseInCost'] = None

        # set to None if opportunity_key (nullable) is None
        # and model_fields_set contains the field
        if self.opportunity_key is None and "opportunity_key" in self.model_fields_set:
            _dict['OpportunityKey'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of BidOpportunity from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "CampaignId": obj.get("CampaignId") if obj.get("CampaignId") is not None else None,
                        "AdGroupId": obj.get("AdGroupId") if obj.get("AdGroupId") is not None else None,
                        "KeywordId": obj.get("KeywordId") if obj.get("KeywordId") is not None else None,
                        "CurrentBid": obj.get("CurrentBid") if obj.get("CurrentBid") is not None else None,
                        "MatchType": obj.get("MatchType") if obj.get("MatchType") is not None else None,
                        "SuggestedBid": obj.get("SuggestedBid") if obj.get("SuggestedBid") is not None else None,
                        "EstimatedIncreaseInImpressions": obj.get("EstimatedIncreaseInImpressions") if obj.get("EstimatedIncreaseInImpressions") is not None else None,
                        "EstimatedIncreaseInClicks": obj.get("EstimatedIncreaseInClicks") if obj.get("EstimatedIncreaseInClicks") is not None else None,
                        "EstimatedIncreaseInCost": obj.get("EstimatedIncreaseInCost") if obj.get("EstimatedIncreaseInCost") is not None else None,
                        "OpportunityKey": obj.get("OpportunityKey") if obj.get("OpportunityKey") is not None else None
        })
        return _obj



from bingads.v13.bulk.entities import *
from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.extensions import *
from .bulk_conversion_goal import BulkConversionGoal

class BulkPagesViewedPerVisitGoal(BulkConversionGoal):
    """ Represents an PagesViewedPerVisit Goal that can be read or written in a bulk file.

    This class exposes the :attr:`pages_viewed_per_visit_goal` property that can be read and written as fields of the
    PagesViewedPerVisit Goal record in a bulk file.

    For more information, see PagesViewedPerVisit Goal at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 pages_viewed_per_visit_goal=None):
        super(BulkPagesViewedPerVisitGoal, self).__init__(conversion_goal = pages_viewed_per_visit_goal)

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.MinimumPagesViewed,
            field_to_csv=lambda c: bulk_str(c.pages_viewed_per_visit_goal.MinimumPagesViewed),
            csv_to_field=lambda c, v: setattr(c.pages_viewed_per_visit_goal, 'MinimumPagesViewed', int(v) if v else None)
        ),
    ]

    @property
    def pages_viewed_per_visit_goal(self):
        """ Defines a PagesViewedPerVisit Goal """

        return self._conversion_goal

    @pages_viewed_per_visit_goal.setter
    def pages_viewed_per_visit_goal(self, pages_viewed_per_visit_goal):
        self._conversion_goal = pages_viewed_per_visit_goal


    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.pages_viewed_per_visit_goal, 'pages_viewed_per_visit_goal')
        super(BulkPagesViewedPerVisitGoal, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkPagesViewedPerVisitGoal._MAPPINGS)

    def process_mappings_from_row_values(self, row_values):
        self.pages_viewed_per_visit_goal = _CAMPAIGN_OBJECT_FACTORY_V13.create('PagesViewedPerVisitGoal')
        super(BulkPagesViewedPerVisitGoal, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkPagesViewedPerVisitGoal._MAPPINGS)




import os
from subprocess import call

dependent_graph = {}
tasks = {}


def dependent_on(*dependent_funcs):
    """ Describe the dependencies of a specific functions.

    :param dependent_funcs: dependent functions
    :return: decorated function
    """

    def dependencies_wrapper(func):
        dependent_graph[func.__name__] = [dependent_func.__name__ for dependent_func in dependent_funcs]
        return func

    return dependencies_wrapper


def task(description=None):
    """ Describe the function be decorated by this task function.

    :param description: the description of decorated function
    :return: decorated function
    """

    if description is None:
        description = ''

    def task_wrapper(func):
        tasks[func.__name__] = description
        return func

    return task_wrapper


def delete_file(file_path):
    """ Delete file specified by path.

    :param file_path: the file path
    :type file_path: str
    :return: None
    """

    try:
        os.remove(file_path)
    except OSError:
        pass


def delete_folder(folder_path):
    """ Delete folder specified by path and all the files under it recursively.

    :param folder_path: the folder path
    :type folder_path: the folder path
    :return: None
    """

    import shutil

    shutil.rmtree(folder_path, ignore_errors=True)


def run_cmd(command):
    """ Run os command.

    :param command: the concrete content of command
    :type command: str
    :return: None
    """

    print(command)
    call(command, shell=True)


def make_execution_plan(func_name):
    """ Make the execution plan for a specific function, consider the dependencies.

    After analyze the dependency graph, use topological sort to give a proper execution order

    :param func_name: the function name to be executed
    :type func_name: str
    :return: the iterable function names, by execution order
    :rtype: iter(str)
    """

    in_degree = {}
    next_tasks = {}

    remaining = [func_name]
    while len(remaining) != 0:
        current = remaining.pop()
        if current not in dependent_graph:
            if current not in in_degree:
                in_degree[current] = 0
            if current not in next_tasks:
                next_tasks[current] = []
            continue
        if current not in in_degree:
            in_degree[current] = 0
        in_degree[current] += len(dependent_graph[current])
        if current not in next_tasks:
            next_tasks[current] = []
        for dependent_func in dependent_graph[current]:
            if dependent_func not in in_degree:
                in_degree[dependent_func] = 0
            if dependent_func not in next_tasks:
                next_tasks[dependent_func] = []
            next_tasks[dependent_func].append(current)
    while len(in_degree) != 0:
        for t in in_degree.keys():
            # can be optimized here, use linear search for easy to implemented and no performance concern currently
            if in_degree[t] == 0:
                t_selected = t
                break
        for t_next in next_tasks[t_selected]:
            in_degree[t_next] -= 1
        in_degree.pop(t_selected)
        next_tasks.pop(t_selected)
        yield t_selected


##########################
# Define Tasks From Here #
##########################


@task('clean the temporary output file')
def clean():
    delete_folder('.tox')
    delete_folder('bingads.egg-info')
    delete_folder('docs/_build')
    delete_folder('dist')
    delete_file('.coverage')


@task('code style check by flake8')
def lint():
    run_cmd('flake8 bingads tests')


@task('run all tests under current interpreter, and print coverage report')
def test():
    run_cmd('coverage run --source bingads -m py.test -v --strict')
    run_cmd('coverage report')


@task('run all unit tests under current interpreter, and print coverage report')
def ut():
    run_cmd('coverage run --source bingads -m py.test -k "not functional" -v --strict')
    run_cmd('coverage report')
	
@task('run all functional tests under current interpreter.')
def ft():
    run_cmd('py.test -k "functional" -v --strict')

@task('run all v13 unit tests under current interpreter, and print coverage report')
def v13_ut():
    run_cmd('coverage run --source bingads -m py.test v13tests/ -k "not functional" -v --strict')
    run_cmd('coverage report')
	
@task('run all v13 functional tests under current interpreter.')
def v13_ft():
    run_cmd('py.test v13tests/ -k "functional" -v --strict')

@task('run tests on all supported interpreters (check tox.ini)')
@dependent_on(clean)
def test_all():
    run_cmd('tox')


@task('generate static html documents, by sphinx, under docs/_build/html')
@dependent_on(clean)
def docs():
    run_cmd('sphinx-apidoc -F -o docs bingads')  # generate API rst file, may need to check-in
    run_cmd('sphinx-build -b html docs docs/_build/html')  # generate html file


@task('make compressed installation package')
@dependent_on(clean)
def dist():
    run_cmd('python setup.py sdist --formats=gztar,zip')


if __name__ == '__main__':
    from sys import argv, exit, modules

    if len(argv) == 1:
        if len(tasks) == 0:
            print('No task defined yet.')
            exit(0)
        max_length = 0
        for key in tasks.keys():
            if len(key) > max_length:
                max_length = len(key)
        format_str = '{0:<' + str(max_length) + '}  --  {1}'
        for key in tasks.keys():
            print(format_str.format(key, tasks[key]))
    elif len(argv) > 2:
        print('Only support one task at a time')
        exit(-1)
    else:
        task_name = argv[1]
        if task_name not in tasks:
            print(str.format("Task: '{0}' not defined", task_name))
            exit(-1)
        task_names = make_execution_plan(task_name)
        for task_name in task_names:
            task = getattr(modules[__name__], task_name)
            task()



from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.extensions import *
from decimal import Decimal


class BulkVideo(_SingleRecordBulkEntity):
    """ Represents a video that can be read or written in a bulk file.

    Properties of this class and of classes that it is derived from, correspond to fields of the Video record in a bulk file.
    For more information, see Video at https://go.microsoft.com/fwlink/?linkid=846127

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, video=None, account_id=None):
        super(BulkVideo, self).__init__()
        self._account_id = account_id
        self._video=video

    @property
    def video(self):
        """
        the Video object
        """
        return self._video

    @video.setter
    def video(self, value):
        self._video = value

    @property
    def account_id(self):
        """ the id of the account which contains the video
        Corresponds to the 'Parent Id' field in the bulk file.

        :rtype: long
        """
        return self._account_id
    
    @account_id.setter
    def account_id(self, value):
        self._account_id = value


    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.video.Id),
            csv_to_field=lambda c, v: setattr(c.video, 'Id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.account_id),
            csv_to_field=lambda c, v: setattr(c, 'account_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: c.video.Status,
            csv_to_field=lambda c, v: setattr(c.video, 'Status', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Description,
            field_to_csv=lambda c: bulk_str(c.video.Description),
            csv_to_field=lambda c, v: setattr(c.video, 'Description', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AspectRatio,
            field_to_csv=lambda c: bulk_str(c.video.AspectRatio),
            csv_to_field=lambda c, v: setattr(c.video, 'AspectRatio', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Url,
            field_to_csv=lambda c: bulk_str(c.video.Url),
            csv_to_field=lambda c, v: setattr(c.video, 'Url', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.SourceUrl,
            field_to_csv=lambda c: bulk_str(c.video.SourceUrl),
            csv_to_field=lambda c, v: setattr(c.video, 'SourceUrl', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ThumbnailUrl,
            field_to_csv=lambda c: bulk_str(c.video.ThumbnailUrl),
            csv_to_field=lambda c, v: setattr(c.video, 'ThumbnailUrl', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.DurationInMillionSeconds,
            field_to_csv=lambda c: bulk_str(c.video.DurationInMilliseconds),
            csv_to_field=lambda c, v: setattr(c.video, 'DurationInMilliseconds', int(v) if v else None)
        )
    ]

    def process_mappings_from_row_values(self, row_values):
        self.video=_CAMPAIGN_OBJECT_FACTORY_V13.create('Video')
        row_values.convert_to_entity(self, BulkVideo._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self.convert_to_values(row_values, BulkVideo._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkVideo, self).read_additional_data(stream_reader)



from .csv_headers import _CsvHeaders
from .mappings import _SimpleBulkMapping
from bingads.v13.bulk import EntityReadException


class _RowValues:
    def __init__(self, mappings=None, columns=None):
        self._mappings = mappings
        self._columns = columns
        if self.mappings is None:
            self._mappings = _CsvHeaders.get_mappings()
        if self.columns is None:
            self._columns = [None] * len(self._mappings)

    def __getitem__(self, key):
        return self.columns[self._mappings[key]]

    def __setitem__(self, key, value):
        self.columns[self._mappings[key]] = value

    def __contains__(self, item):
        return item in self.mappings

    def __len__(self):
        return len(self.mappings)

    def __str__(self):
        return u'{' + u', '.join([u'{0}:{1}'.format(k, self.columns[v]) for (k, v) in self.mappings.items()]) + u'}'

    def convert_to_entity(self, entity, bulk_mappings):
        for mapping in bulk_mappings:
            try:
                mapping.convert_to_entity(self, entity)
            except Exception as ex:
                raise self._create_entity_read_exception(entity, mapping, ex)

    def _create_entity_read_exception(self, entity, mapping, ex):
        entity_type = str(type(entity))

        if isinstance(mapping, _SimpleBulkMapping):
            message = "Couldn't parse column {0} of {1} entity: {2}".format(
                mapping.header,
                entity_type,
                str(ex)
            )
        else:
            message = "Couldn't parse {0} entity: {1}".format(entity_type, str(ex))
        message += " See ColumnValues for detailed row information and InnerException for error details."
        message += u' row values: {0}'.format(self)

        return EntityReadException(message=message, row_values=str(self), inner_exception=ex)

    def try_get_value(self, header):
        if header not in self.mappings:
            return False, None
        return True, self[header]

    def to_dict(self):
        return dict([(k, self.columns[v]) for (k, v) in self.mappings.items()])

    @property
    def mappings(self):
        return self._mappings

    @property
    def columns(self):
        return self._columns



from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.bulk.entities import *
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping, _ComplexBulkMapping
from bingads.v13.internal.extensions import *


def bidding_scheme_to_csv(bulk_keyword, row_values):
    bid_strategy_type = field_to_csv_BidStrategyType(bulk_keyword.keyword)
    if not bid_strategy_type:
        return
    row_values[_StringTable.BidStrategyType] = bid_strategy_type
    if bid_strategy_type == 'InheritFromParent' \
        and hasattr(bulk_keyword.keyword.BiddingScheme, 'InheritedBidStrategyType'):
        row_values[_StringTable.InheritedBidStrategyType] = bulk_keyword.keyword.BiddingScheme.InheritedBidStrategyType


def csv_to_bidding_scheme(row_values, bulk_keyword):
    success, bid_strategy_type = row_values.try_get_value(_StringTable.BidStrategyType)
    if not success or not bid_strategy_type:
        return
    csv_to_field_BidStrategyType(bulk_keyword.keyword, bid_strategy_type)
    if bid_strategy_type == 'InheritFromParent':
        bulk_keyword.keyword.BiddingScheme.Type = "InheritFromParent"
        success, inherited_bid_strategy_type = row_values.try_get_value(_StringTable.InheritedBidStrategyType)
        if success and inherited_bid_strategy_type != '':
            bulk_keyword.keyword.BiddingScheme.InheritedBidStrategyType = inherited_bid_strategy_type
        elif hasattr(bulk_keyword.keyword.BiddingScheme, 'InheritedBidStrategyType'):
            bulk_keyword.keyword.BiddingScheme.InheritedBidStrategyType = None
    else:
        bulk_keyword.keyword.BiddingScheme.Type = bid_strategy_type


class BulkKeyword(_SingleRecordBulkEntity):
    """ Represents a keyword that can be read or written in a bulk file.

    This class exposes the :attr:`keyword` property that can be read and written as fields of the Keyword record in a bulk file.
    Properties of this class and of classes that it is derived from, correspond to fields of the Keyword record in a bulk file.
    For more information, see Keyword at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, ad_group_id=None, campaign_name=None, ad_group_name=None, keyword=None):
        super(BulkKeyword, self).__init__()
        self._ad_group_id = ad_group_id
        self._keyword = keyword
        self._campaign_name = campaign_name
        self._ad_group_name = ad_group_name
        self._quality_score_data = None
        self._bid_suggestions = None

    @property
    def ad_group_id(self):
        """ The identifier of the ad group that contains the keyword.

        Corresponds to the 'Parent Id' field in the bulk file.

        :rtype: int
        """

        return self._ad_group_id

    @ad_group_id.setter
    def ad_group_id(self, ad_group_id):
        self._ad_group_id = ad_group_id

    @property
    def keyword(self):
        """ Defines a keyword within an ad group.

        See Keyword at https://docs.microsoft.com/en-us/bingads/campaign-management-service/keyword?view=bingads-13
        """

        return self._keyword

    @keyword.setter
    def keyword(self, keyword):
        self._keyword = keyword

    @property
    def campaign_name(self):
        """ The name of the campaign that contains the keyword.

        Corresponds to the 'Campaign' field in the bulk file.

        :rtype: str
        """

        return self._campaign_name

    @campaign_name.setter
    def campaign_name(self, campaign_name):
        self._campaign_name = campaign_name

    @property
    def ad_group_name(self):
        """ The name of the ad group that contains the keyword.

        Corresponds to the 'Ad Group' field in the bulk file.

        :rtype: str
        """

        return self._ad_group_name

    @ad_group_name.setter
    def ad_group_name(self, ad_group_name):
        self._ad_group_name = ad_group_name

    @property
    def quality_score_data(self):
        """ The quality score data for the keyword.

        :rtype: QualityScoreData
        """

        return self._quality_score_data

    @property
    def bid_suggestions(self):
        """ The bid suggestion data for the keyword.

        :rtype: BidSuggestionData
        """

        return self._bid_suggestions

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: bulk_str(c.keyword.Status),
            csv_to_field=lambda c, v: csv_to_field_enum(c.keyword, v, 'Status', KeywordStatus)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.keyword.Id),
            csv_to_field=lambda c, v: setattr(
                c.keyword,
                'Id',
                int(v) if v else None
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.ad_group_id),
            csv_to_field=lambda c, v: setattr(
                c,
                '_ad_group_id',
                int(v) if v else None
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.Campaign,
            field_to_csv=lambda c: c.campaign_name,
            csv_to_field=lambda c, v: setattr(c, '_campaign_name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AdGroup,
            field_to_csv=lambda c: c.ad_group_name,
            csv_to_field=lambda c, v: setattr(c, '_ad_group_name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Keyword,
            field_to_csv=lambda c: c.keyword.Text,
            csv_to_field=lambda c, v: setattr(c.keyword, 'Text', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.EditorialStatus,
            field_to_csv=lambda c: bulk_str(c.keyword.EditorialStatus),
            csv_to_field=lambda c, v: csv_to_field_enum(c.keyword, v, 'EditorialStatus', KeywordEditorialStatus)
        ),
        _SimpleBulkMapping(
            header=_StringTable.MatchType,
            field_to_csv=lambda c: bulk_str(c.keyword.MatchType),
            csv_to_field=lambda c, v: csv_to_field_enum(c.keyword, v, 'MatchType', MatchType)
        ),
        _SimpleBulkMapping(
            header=_StringTable.DestinationUrl,
            field_to_csv=lambda c: bulk_optional_str(c.keyword.DestinationUrl, c.keyword.Id),
            csv_to_field=lambda c, v: setattr(
                c.keyword,
                'DestinationUrl',
                v if v else None
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.Bid,
            field_to_csv=lambda c: keyword_bid_bulk_str(c.keyword.Bid, c.keyword.Id),
            csv_to_field=lambda c, v: setattr(
                c.keyword,
                'Bid',
                parse_keyword_bid(v)
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.Param1,
            field_to_csv=lambda c: bulk_optional_str(c.keyword.Param1, c.keyword.Id),
            csv_to_field=lambda c, v: setattr(
                c.keyword,
                'Param1',
                v if v else ''
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.Param2,
            field_to_csv=lambda c: bulk_optional_str(c.keyword.Param2, c.keyword.Id),
            csv_to_field=lambda c, v: setattr(
                c.keyword,
                'Param2',
                v if v else ''
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.Param3,
            field_to_csv=lambda c: bulk_optional_str(c.keyword.Param3, c.keyword.Id),
            csv_to_field=lambda c, v: setattr(
                c.keyword,
                'Param3',
                v if v else ''
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.FinalUrl,
            field_to_csv=lambda c: field_to_csv_Urls(c.keyword.FinalUrls, c.keyword.Id),
            csv_to_field=lambda c, v: csv_to_field_Urls(c.keyword.FinalUrls, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FinalMobileUrl,
            field_to_csv=lambda c: field_to_csv_Urls(c.keyword.FinalMobileUrls, c.keyword.Id),
            csv_to_field=lambda c, v: csv_to_field_Urls(c.keyword.FinalMobileUrls, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.TrackingTemplate,
            field_to_csv=lambda c: bulk_optional_str(c.keyword.TrackingUrlTemplate, c.keyword.Id),
            csv_to_field=lambda c, v: setattr(c.keyword, 'TrackingUrlTemplate', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.CustomParameter,
            field_to_csv=lambda c: field_to_csv_UrlCustomParameters(c.keyword),
            csv_to_field=lambda c, v: csv_to_field_UrlCustomParameters(c.keyword, v)
        ),
        _ComplexBulkMapping(bidding_scheme_to_csv, csv_to_bidding_scheme),
        _SimpleBulkMapping(
            header=_StringTable.FinalUrlSuffix,
            field_to_csv=lambda c: bulk_optional_str(c.keyword.FinalUrlSuffix, c.keyword.Id),
            csv_to_field=lambda c, v: setattr(c.keyword, 'FinalUrlSuffix', v if v else None)
        ),
    ]

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self._keyword, 'keyword')
        self.convert_to_values(row_values, BulkKeyword._MAPPINGS)
        if not exclude_readonly_data:
            QualityScoreData.write_to_row_values_if_not_null(self.quality_score_data, row_values)

    def process_mappings_from_row_values(self, row_values):
        self._keyword = _CAMPAIGN_OBJECT_FACTORY_V13.create('Keyword')
        row_values.convert_to_entity(self, BulkKeyword._MAPPINGS)
        self._quality_score_data = QualityScoreData.read_from_row_values_or_null(row_values)

    def read_additional_data(self, stream_reader):
        success, next_bid_suggestion = stream_reader.try_read(BulkKeywordBidSuggestion)

        while success:
            if self._bid_suggestions is None:
                self._bid_suggestions = BidSuggestionData()

            if isinstance(next_bid_suggestion, BulkKeywordBestPositionBid):
                self._bid_suggestions.best_position = next_bid_suggestion
            elif isinstance(next_bid_suggestion, BulkKeywordMainLineBid):
                self._bid_suggestions.main_line = next_bid_suggestion
            elif isinstance(next_bid_suggestion, BulkKeywordFirstPageBid):
                self._bid_suggestions.first_page = next_bid_suggestion

            success, next_bid_suggestion = stream_reader.try_read(BulkKeywordBidSuggestion)

    def write_additional_data(self, row_writer):
        if self.bid_suggestions is not None:
            BulkKeywordBidSuggestion.write_if_not_null(self.bid_suggestions.best_position, row_writer)
            BulkKeywordBidSuggestion.write_if_not_null(self.bid_suggestions.main_line, row_writer)
            BulkKeywordBidSuggestion.write_if_not_null(self.bid_suggestions.first_page, row_writer)



# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.reporting.account_through_ad_group_report_scope import AccountThroughAdGroupReportScope
from openapi_client.models.reporting.report_aggregation import ReportAggregation
from openapi_client.models.reporting.report_format import ReportFormat
from openapi_client.models.reporting.report_time import ReportTime
from openapi_client.models.reporting.user_location_performance_report_column import UserLocationPerformanceReportColumn
from openapi_client.models.reporting.user_location_performance_report_filter import UserLocationPerformanceReportFilter
from typing_extensions import Self

class UserLocationPerformanceReportRequest(BaseModel):
    """
    UserLocationPerformanceReportRequest
    """ # noqa: E501
    scope: Optional[AccountThroughAdGroupReportScope] = Field(default=None, alias="Scope")
    filter: Optional[UserLocationPerformanceReportFilter] = Field(default=None, alias="Filter")
    time: Optional[ReportTime] = Field(default=None, alias="Time")
    aggregation: Optional[ReportAggregation] = Field(default=None, alias="Aggregation")
    columns: Optional[List[Optional[UserLocationPerformanceReportColumn]]] = Field(default=None, alias="Columns")
    report_name: Optional[StrictStr] = Field(default=None, alias="ReportName")
    format: Optional[ReportFormat] = Field(default=None, alias="Format")
    return_only_complete_data: Optional[StrictBool] = Field(default=None, alias="ReturnOnlyCompleteData")
    exclude_report_header: Optional[StrictBool] = Field(default=None, alias="ExcludeReportHeader")
    exclude_report_footer: Optional[StrictBool] = Field(default=None, alias="ExcludeReportFooter")
    exclude_column_headers: Optional[StrictBool] = Field(default=None, alias="ExcludeColumnHeaders")
    format_version: Optional[StrictStr] = Field(default=None, alias="FormatVersion")
    type: Optional[StrictStr] = Field(default='UserLocationPerformanceReportRequest', alias="Type")
    __properties: ClassVar[List[str]] = ["Scope", "Filter", "Time", "Aggregation", "Columns", "ReportName", "Format", "ReturnOnlyCompleteData", "ExcludeReportHeader", "ExcludeReportFooter", "ExcludeColumnHeaders", "FormatVersion", "Type"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	
    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())
	
    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of UserLocationPerformanceReportRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of scope
        if self.scope:
            _dict['Scope'] = self.scope.to_dict()
        # override the default output from pydantic by calling `to_dict()` of filter
        if self.filter:
            _dict['Filter'] = self.filter.to_dict()
        # override the default output from pydantic by calling `to_dict()` of time
        if self.time:
            _dict['Time'] = self.time.to_dict()
        # set to None if scope (nullable) is None
        # and model_fields_set contains the field
        if self.scope is None and "scope" in self.model_fields_set:
            _dict['Scope'] = None

        # set to None if filter (nullable) is None
        # and model_fields_set contains the field
        if self.filter is None and "filter" in self.model_fields_set:
            _dict['Filter'] = None

        # set to None if time (nullable) is None
        # and model_fields_set contains the field
        if self.time is None and "time" in self.model_fields_set:
            _dict['Time'] = None

        # set to None if aggregation (nullable) is None
        # and model_fields_set contains the field
        if self.aggregation is None and "aggregation" in self.model_fields_set:
            _dict['Aggregation'] = None

        # set to None if columns (nullable) is None
        # and model_fields_set contains the field
        if self.columns is None and "columns" in self.model_fields_set:
            _dict['Columns'] = None

        # set to None if report_name (nullable) is None
        # and model_fields_set contains the field
        if self.report_name is None and "report_name" in self.model_fields_set:
            _dict['ReportName'] = None

        # set to None if format (nullable) is None
        # and model_fields_set contains the field
        if self.format is None and "format" in self.model_fields_set:
            _dict['Format'] = None

        # set to None if return_only_complete_data (nullable) is None
        # and model_fields_set contains the field
        if self.return_only_complete_data is None and "return_only_complete_data" in self.model_fields_set:
            _dict['ReturnOnlyCompleteData'] = None

        # set to None if exclude_report_header (nullable) is None
        # and model_fields_set contains the field
        if self.exclude_report_header is None and "exclude_report_header" in self.model_fields_set:
            _dict['ExcludeReportHeader'] = None

        # set to None if exclude_report_footer (nullable) is None
        # and model_fields_set contains the field
        if self.exclude_report_footer is None and "exclude_report_footer" in self.model_fields_set:
            _dict['ExcludeReportFooter'] = None

        # set to None if exclude_column_headers (nullable) is None
        # and model_fields_set contains the field
        if self.exclude_column_headers is None and "exclude_column_headers" in self.model_fields_set:
            _dict['ExcludeColumnHeaders'] = None

        # set to None if format_version (nullable) is None
        # and model_fields_set contains the field
        if self.format_version is None and "format_version" in self.model_fields_set:
            _dict['FormatVersion'] = None

        # set to None if type (nullable) is None
        # and model_fields_set contains the field
        if self.type is None and "type" in self.model_fields_set:
            _dict['Type'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of UserLocationPerformanceReportRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Scope": AccountThroughAdGroupReportScope.from_dict(obj["Scope"]) if obj.get("Scope") is not None else None,
                        "Filter": UserLocationPerformanceReportFilter.from_dict(obj["Filter"]) if obj.get("Filter") is not None else None,
                        "Time": ReportTime.from_dict(obj["Time"]) if obj.get("Time") is not None else None,
                        "Aggregation": obj.get("Aggregation") if obj.get("Aggregation") is not None else None,
                        "Columns": obj.get("Columns"),
                        "ReportName": obj.get("ReportName") if obj.get("ReportName") is not None else None,
                        "Format": obj.get("Format") if obj.get("Format") is not None else None,
                        "ReturnOnlyCompleteData": obj.get("ReturnOnlyCompleteData") if obj.get("ReturnOnlyCompleteData") is not None else None,
                        "ExcludeReportHeader": obj.get("ExcludeReportHeader") if obj.get("ExcludeReportHeader") is not None else None,
                        "ExcludeReportFooter": obj.get("ExcludeReportFooter") if obj.get("ExcludeReportFooter") is not None else None,
                        "ExcludeColumnHeaders": obj.get("ExcludeColumnHeaders") if obj.get("ExcludeColumnHeaders") is not None else None,
                        "FormatVersion": obj.get("FormatVersion") if obj.get("FormatVersion") is not None else None,
                        "Type": obj.get("Type") if obj.get("Type") is not None else 'UserLocationPerformanceReportRequest'
        })
        return _obj



from .enums import ResultFileType
from .entities.bulk_entity import BulkEntity
from bingads.v13.internal.bulk.stream_reader import _BulkStreamReader
from bingads.v13.internal.bulk.record_reader import _BulkRecordReader
from bingads.v13.internal.bulk.object_reader import _BulkRowsObjectReader
from bingads.v13.internal.bulk.entities.multi_record_bulk_entity import _MultiRecordBulkEntity

class _BulkEntityReader(object):
    """ Provides a method to read bulk entities from a bulk file and make them accessible as an enumerable list.

    For more information about the Bulk File Schema, see https://go.microsoft.com/fwlink/?linkid=846127.
    """

    def __init__(self, record_reader, is_for_full_download):
        """ Initializes a new instance of this class with the specified file details.

        :param record_reader: The bulk record reader.
        :type record_reader: _BulkRecordReader
        :param is_for_full_download: si for full download or not.
        :type encoding: boolean
        """

        self._is_for_full_download = is_for_full_download
        self._entities_iterator = None
        self._bulk_record_reader = record_reader
        self._bulk_record_reader.__enter__()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self._entities_iterator = None
        self._bulk_record_reader.__exit__(exc_type, exc_value, traceback)

    def __iter__(self):
        return self

    def __next__(self):
        return self.next()

    def close(self):
        self.__exit__(None, None, None)

    def next(self):
        return self.read_next_entity()

    def read_next_entity(self):
        """ Reads next entity from the bulk file.

        :return: next entity
        :rtype: BulkEntity
        """

        if self._entities_iterator is None:
            self._entities_iterator = self.read_entities()

        return next(self._entities_iterator)

    def read_entities(self):
        """ Gets an enumerable list of bulk entities that were read from the file.

        :return: an generator over the entities
        :rtype: collections.Iterator[BulkEntity]
        """

        next_batch = self._read_next_batch()

        while next_batch is not None:
            for entity in next_batch:
                yield entity
            next_batch = self._read_next_batch()

    def _read_next_batch(self):
        """ Reads next batch of entities from the file.

        Batch means a set of related entities.
        It can be one :class:`._SingleRecordBulkEntity`, one :class:`._MultiRecordBulkEntity` containing its child
        entities or a set of related child entities (for example several :class:`.BulkSiteLink`s logically belonging
        to the same SiteLink Ad Extension.

        :return: Next batch of entities
        :rtype: _SingleRecordBulkEntity or _MultiRecordBulkEntity
        """

        next_object = self._bulk_record_reader.read()

        if next_object is None:
            return None
        if next_object.can_enclose_in_multiline_entity:
            multi_record_entity = next_object.enclose_in_multiline_entity()
            multi_record_entity.read_related_data_from_stream(self._bulk_record_reader)
            if self._is_for_full_download:
                return [multi_record_entity]
            return self._extract_child_entities_if_needed(multi_record_entity)
        if isinstance(next_object, BulkEntity):
            return [next_object]
        raise NotImplementedError()

    def _extract_child_entities_if_needed(self, entity):
        # If the entity is a MultiLine entity and it has all child objects (delete all row was present), just return it
        if not isinstance(entity, _MultiRecordBulkEntity) or entity.all_children_are_present:
            yield entity
        else:
            # If not all child objects are present (there was no delete all row and we only have part of the MultiLine entity), return child object individually
            for child_generator in (
                    self._extract_child_entities_if_needed(child_entity)
                    for child_entity in entity.child_entities):
                for child in child_generator:
                    yield child


class BulkRowsReader(_BulkEntityReader):
    def __init__(self, csv_rows):
        super(BulkRowsReader, self).__init__(_BulkRecordReader(_BulkRowsObjectReader(csv_rows)), False)

class BulkFileReader(_BulkEntityReader):
    """ Provides a method to read bulk entities from a bulk file and make them accessible as an enumerable list.

    For more information about the Bulk File Schema, see https://go.microsoft.com/fwlink/?linkid=846127.
    """

    def __init__(self,
                 file_path,
                 file_type='Csv',
                 result_file_type=ResultFileType.full_download,
                 encoding='utf-8-sig'):
        """ Initializes a new instance of this class with the specified file details.

        :param file_path: The path of the bulk file to read.
        :type file_path: str
        :param file_type: The bulk file type.
        :type file_type: str
        :param result_file_type: The result file type.
        :type result_file_type: ResultFileType
        :param encoding: The encoding of bulk file.
        :type encoding: str
        """
        super(BulkFileReader, self).__init__(_BulkStreamReader(file_path=file_path, file_type=file_type, encoding=encoding),  result_file_type is ResultFileType.full_download)
        self._file_path = file_path
        self._file_type = file_type
        self._result_file_type = result_file_type
        self._encoding = encoding

        self._is_for_full_download = result_file_type is ResultFileType.full_download
        
    @property
    def file_path(self):
        """ The path of the bulk file to read.

        :rtype: str
        """

        return self._file_path

    @property
    def file_type(self):
        """ The bulk file type.

        :rtype: str
        """

        return self._file_type

    @property
    def result_file_type(self):
        """ The result file type.

        :rtype: ResultFileType
        """

        return self._result_file_type

    @property
    def encoding(self):
        """ The encoding of bulk file.

        :rtype: str
        """

        return self._encoding



import pkg_resources

_SERVICE_LIST = ['adinsight', 'bulk', 'campaignmanagement', 'customerbilling', 'customermanagement', 'reporting']

SERVICE_INFO_DICT_V13 = {}

for service in _SERVICE_LIST:
    SERVICE_INFO_DICT_V13[(service, 'production')] = 'file:///' + pkg_resources.resource_filename('bingads', 'v13/proxies/production/%s_service.xml' % (service))
    SERVICE_INFO_DICT_V13[(service, 'sandbox')] = 'file:///' + pkg_resources.resource_filename('bingads', 'v13/proxies/sandbox/%s_service.xml' % (service))

SERVICE_INFO_DICT = {13: SERVICE_INFO_DICT_V13}



from bingads.exceptions import SdkException

class Report:
    
    def __init__(self):
        self._report_header = None
        self._report_iterator = None
        pass
    
    @property
    def report_name(self):
        return self._report_header.report_name

    @property
    def last_completed_available_date(self):
        return self._report_header.last_completed_available_date

    @property
    def record_count(self):
        return self._report_header.record_count

    @property
    def time_zone(self):
        return self._report_header.time_zone

#    @property
#    def report_filter(self):
#        return self._report_header.report_filter

    @property
    def potential_incomplete_data(self):
        return self._report_header.potential_incomplete_data

    @property
    def report_aggregation(self):
        return self._report_header.report_aggregation

    @property
    def report_time_start(self):
        return self._report_header.report_time_start
    
    @property
    def report_time_end(self):
        return self._report_header.report_time_end

    @property
    def report_columns(self):
        return self._report_header.report_columns

    @property
    def report_records(self):
        return self._report_iterator
    
    def close(self):
        pass


class InvalidReportFormatException(SdkException):

    def __init__(self, format):
        """ Initializes a new instance of this class with the specified error messages.

        :param format: The format of the report file to read.
        :type format: str
        """
        super(InvalidReportFormatException, self).__init__(str.format("Report format {0} is not supported.", format))

class InvalidReportContentException(SdkException):

    def __init__(self):
        super(InvalidReportContentException, self).__init__("Report content is invalid.")


class InvalidReportColumnException(SdkException):
    """ This exception is thrown if trying to retrieve inexistent column """

    def __init__(self, column_name):
        """ Initializes a new instance of this class with the specified error messages.

        :param column_name: The column name to be retrieved from the report.
        :type column_name: str
        """
        super(InvalidReportColumnException, self).__init__(str.format("Field: {0} does not exist in report.", column_name))


from bingads.manifest import BULK_FORMAT_VERSION_6

from bingads.v13.internal.bulk.bulk_object_factory import _BulkObjectFactory
from bingads.v13.internal.bulk.csv_writer import _CsvWriter
from bingads.v13.internal.bulk.csv_headers import _CsvHeaders
from bingads.v13.internal.bulk.row_values import _RowValues
from bingads.v13.internal.bulk.string_table import _StringTable


class _BulkObjectWriter():
    def __init__(self, file_path, file_type):
        self._file_path = file_path

        if file_type == 'Csv':
            self._delimiter = ','
        elif file_type == 'Tsv':
            self._delimiter = '\t'
        else:
            raise ValueError('Invalid file_type provided: {0}'.format(file_type))

        self._csv_writer = _CsvWriter(self.file_path, delimiter=self._delimiter)
        self._csv_writer.__enter__()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._csv_writer.__exit__(exc_type, exc_val, exc_tb)

    def close(self):
        self.__exit__(None, None, None)

    def write_file_metadata(self):
        self.write_headers()
        self.write_format_version()

    def write_headers(self):
        self._csv_writer.writerow(_CsvHeaders.HEADERS)

    def write_format_version(self):
        version_row = _RowValues()
        version_row[_StringTable.Type] = _StringTable.SemanticVersion
        version_row[_StringTable.Name] = BULK_FORMAT_VERSION_6
        self._csv_writer.writerow(version_row.columns)

    def write_object_row(self, bulk_object, exclude_readonly_data=False):
        values = _RowValues()
        bulk_object.write_to_row_values(values, exclude_readonly_data)
        values[_StringTable.Type] = _BulkObjectFactory.get_bulk_row_type(bulk_object)
        self._csv_writer.writerow(values.columns)

    @property
    def file_path(self):
        return self._file_path

    @property
    def delimiter(self):
        return self._delimiter



from urllib.parse import parse_qs, urlparse, quote_plus
import json
from datetime import datetime, timedelta
import requests
from openapi_client.exceptions import ApiException

PRODUCTION = 'production'
SANDBOX = 'sandbox'
MSADS_MANAGE = 'msads.manage'
ADS_MANAGE = 'ads.manage'
BINGADS_MANAGE = 'bingads.manage'
MSA_PROD = 'msa.prod'


class OAuthTokenRequestException(ApiException):
    """OAuth token request exception that includes error details from the OAuth service."""

    def __init__(self, error_code, error_description):
        super(OAuthTokenRequestException, self).__init__(
            status=None,
            reason=f"OAuth token request failed. Error: {error_code}. Description: {error_description}"
        )
        self.error_code = error_code
        self.error_description = error_description


class AuthorizationData:
    """ Represents a user who intends to access the corresponding customer and account.

    An instance of this class is required to authenticate with Bing Ads REST API services.
    """

    def __init__(self,
                 account_id=None,
                 customer_id=None,
                 developer_token=None,
                 authentication=None):
        """ Initialize an instance of this class.

        :param account_id: The identifier of the account that owns the entities in the request.
        :type account_id: int
        :param customer_id: The identifier of the customer that owns the account.
        :type customer_id: int
        :param developer_token: The Bing Ads developer access token.
        :type developer_token: str
        :param authentication: An object representing the authentication method.
        :type authentication: Authentication
        """

        self._account_id = account_id
        self._customer_id = customer_id
        self._developer_token = developer_token
        self._authentication = authentication

    @property
    def account_id(self):
        """ The identifier of the account that owns the entities in the request.
        :rtype: int
        """
        return self._account_id

    @property
    def customer_id(self):
        """ The identifier of the customer that owns the account.
        :rtype: int
        """
        return self._customer_id

    @property
    def developer_token(self):
        """ The Bing Ads developer access token.
        :rtype: str
        """
        return self._developer_token

    @property
    def authentication(self):
        """ An object representing the authentication method.
        :rtype: Authentication
        """
        return self._authentication

    @account_id.setter
    def account_id(self, account_id):
        self._account_id = account_id

    @customer_id.setter
    def customer_id(self, customer_id):
        self._customer_id = customer_id

    @developer_token.setter
    def developer_token(self, developer_token):
        self._developer_token = developer_token

    @authentication.setter
    def authentication(self, authentication):
        self._authentication = authentication


class Authentication(object):
    """ The base class for all authentication classes. """

    def enrich_headers(self, headers):
        """ Gets the required header elements for the corresponding REST API call.

        :return: Dictionary containing authentication headers
        :rtype: dict
        """
        raise NotImplementedError()


class PasswordAuthentication(Authentication):
    """ Represents a legacy authentication method using user name and password. """

    def __init__(self, user_name, password):
        """ Initialize an instance of this class.

        :param user_name: The Bing Ads user's sign-in user name.
        :type user_name: str
        :param password: The Bing Ads user's sign-in password.
        :type password: str
        """
        self._user_name = user_name
        self._password = password

    @property
    def user_name(self):
        """ The Bing Ads user's sign-in user name.
        :rtype: str
        """
        return self._user_name

    @property
    def password(self):
        """ The Bing Ads user's sign-in password.
        :rtype: str
        """
        return self._password

    def enrich_headers(self, headers):
        """ Gets the username/password authentication headers.
        :return: Dictionary containing authentication headers
        :rtype: dict
        """
        auth_headers = {
            'UserName': self.user_name,
            'Password': self.password
        }
        headers.update(auth_headers)


class OAuthTokens:
    """ Contains information about OAuth access tokens received from the Microsoft Account authorization service. """

    def __init__(self, access_token=None, access_token_expires_in_seconds=None, refresh_token=None, response_json = None):
        """ Initialize an instance of this class.

        :param access_token: OAuth access token for authorization
        :type access_token: str
        :param access_token_expires_in_seconds: The access token expiration time in seconds
        :type access_token_expires_in_seconds: int
        :param refresh_token: OAuth refresh token that can be used to refresh an access token
        :type refresh_token: str
        """
        self._access_token = access_token
        self._access_token_expires_in_seconds = access_token_expires_in_seconds
        self._refresh_token = refresh_token
        self._response_json = response_json
        self._access_token_received_datetime = datetime.utcnow()

    @property
    def access_token_received_datetime(self):
        """ The datetime when access token was received

        :rtype: datetime
        """
        return self._access_token_received_datetime

    @property
    def access_token(self):
        """ OAuth access token that will be used for authorization in the Bing Ads services.

        :rtype: str
        """
        return self._access_token

    @property
    def access_token_expires_in_seconds(self):
        """ Expiration time for the corresponding access token in seconds.

        :rtype: int
        """
        return self._access_token_expires_in_seconds

    @property
    def access_token_expired(self):
        """ Whether the access token has been expired.

        :rtype: bool
        """
        return self.access_token_expires_in_seconds is not None and \
            self.access_token_expires_in_seconds > 0 and \
            datetime.utcnow() > self.access_token_received_datetime + timedelta(
                seconds=self.access_token_expires_in_seconds)

    @property
    def refresh_token(self):
        """ OAuth refresh token that can be user to refresh an access token.

        :rtype: str
        """
        return self._refresh_token

    @property
    def response_json(self):
        """ OAuth whole attribute that got along with access token.

        :rtype: dictionary
        """
        return self._response_json


class OAuthAuthorization(Authentication):
    """ The abstract base class for all OAuth authentication classes.

    You can use this class to dynamically instantiate a derived OAuth authentication class at run time.
    This class cannot be instantiated, and instead you should use either :class:`.OAuthDesktopMobileAuthCodeGrant`,
    :class:`.OAuthDesktopMobileImplicitGrant`, :class:`.OAuthWebAuthCodeGrant`, which extend this class.
    """

    def __init__(self, client_id, oauth_tokens=None, env=PRODUCTION, oauth_scope=MSADS_MANAGE, tenant='common',
                 use_msa_prod=True):
        """ Initialize an instance of this class.

        :param client_id: The client identifier corresponding to your registered application.
        :type client_id: str
        :param oauth_tokens: OAuth token information
        :type oauth_tokens: OAuthTokens
        """
        if client_id is None:
            raise ValueError('Client id cannot be None.')
        self._client_id = client_id
        self._oauth_tokens = oauth_tokens
        self._state = None
        self.environment = env
        self._oauth_scope = MSA_PROD if env == SANDBOX and use_msa_prod else oauth_scope
        self._tenant = tenant

    def enrich_headers(self, headers):
        """ Gets the OAuth Bearer token authentication headers.
        :return: Dictionary containing authentication headers
        :rtype: dict
        """
        if self.oauth_tokens is None or self.oauth_tokens.access_token is None:
            raise ValueError("OAuth access token hasn't been requested.")
        auth_headers =  {
            'Authorization': 'Bearer ' + self.oauth_tokens.access_token
        }
        headers.update(auth_headers)

    @property
    def tenant(self):
        """ tenant
        :rtype: str
        """
        return self._tenant

    @property
    def client_id(self):
        """ The client identifier corresponding to your registered application.

        For more information about using a client identifier for authentication, see the
        Client Password Authentication section of the OAuth 2.0 spec at
        https://tools.ietf.org/html/rfc6749#section-4.1

        :rtype: str
        """
        return self._client_id

    @property
    def oauth_tokens(self):
        """ Contains information about OAuth access tokens received from the Microsoft Account authorization service.

        :rtype: OAuthTokens
        """
        return self._oauth_tokens

    @property
    def state(self):
        """ An opaque value used by the client to maintain state between the request and callback
        :rtype: str
        """
        return self._state

    @state.setter
    def state(self, value):
        """ An opaque value used by the client to maintain state between the request and callback
        :rtype: str
        """
        self._state = value

    @property
    def redirection_uri(self):
        """ The URI to which the user of the app will be redirected after receiving user consent.

        :rtype: str
        """
        raise NotImplementedError()

    def get_authorization_endpoint(self):
        """ Gets the Microsoft Account authorization endpoint where the user should be navigated to give his or her consent.

        :return: The Microsoft Account authorization endpoint.
        :rtype: str
        """
        raise NotImplementedError()


class OAuthWithAuthorizationCode(OAuthAuthorization):
    """ Represents a proxy to the Microsoft account authorization service.

    Implementation of the authorization code grant flow for Managing User Authentication with OAuth.
    This is a standard OAuth 2.0 flow defined in the Authorization Code Grant section of the OAuth 2.0 spec
    at https://tools.ietf.org/html/rfc6749#section-4.1.
    """

    def __init__(self, client_id, client_secret, redirection_uri, token_refreshed_callback=None,
                 oauth_tokens=None, env=PRODUCTION, oauth_scope=MSADS_MANAGE, tenant="common", use_msa_prod=True):
        """ Initialize an instance of this class.

        :param client_id: The client identifier corresponding to your registered application.
        :type client_id: str
        :param client_secret: The client secret for your registered application.
        :type client_secret: str
        :param redirection_uri: The URI to which your client browser will be redirected after receiving user consent.
        :type redirection_uri: str
        :param token_refreshed_callback: Optional callback function when oauth_tokens are refreshed.
        :type token_refreshed_callback: OAuthTokens->None or None
        """
        super(OAuthWithAuthorizationCode, self).__init__(
            client_id=client_id,
            oauth_tokens=oauth_tokens,
            env=env,
            oauth_scope=oauth_scope,
            tenant=tenant,
            use_msa_prod=use_msa_prod
        )
        self._client_secret = client_secret
        self._redirection_uri = redirection_uri
        self._token_refreshed_callback = token_refreshed_callback

    def get_authorization_endpoint(self):
        """ Gets the Microsoft Account authorization endpoint where the user should be navigated to give consent.

        :return: The Microsoft Account authorization endpoint.
        :rtype: str
        """
        endpoint_url = _UriOAuthService.AUTHORIZE_URI[(self.environment, self._oauth_scope)]
        if self.environment == PRODUCTION and (self._oauth_scope == MSADS_MANAGE or self._oauth_scope == ADS_MANAGE):
            endpoint_url = endpoint_url.replace('common', self.tenant)

        endpoint = str.format(
            endpoint_url,
            self._client_id,
            'code',
            quote_plus(self._redirection_uri)
        )

        return endpoint if self.state is None else endpoint + '&state=' + self.state

    def request_oauth_tokens_by_response_uri(self, response_uri, **kwargs):
        """ Request OAuth tokens using the authorization response URI.

        :param response_uri: The response redirection URI.
        :type response_uri: str
        :return: OAuth tokens
        :rtype: OAuthTokens
        """
        parameters = parse_qs(urlparse(response_uri).query)
        if 'code' not in parameters or len(parameters['code']) == 0:
            raise ValueError(
                "Uri passed doesn't contain code param. "
                "Please make sure the uri has a code in it, for example http://myurl.com?code=123"
            )
        code = parameters['code'][0]

        self._oauth_tokens = _UriOAuthService.get_access_token(
            client_id=self.client_id,
            client_secret=self.client_secret,
            redirect_uri=self.redirection_uri,
            grant_type='authorization_code',
            environment=self.environment,
            code=code,
            oauth_scope=self._oauth_scope,
            tenant=self.tenant,
            **kwargs
        )
        if self.token_refreshed_callback is not None:
            self.token_refreshed_callback(self.oauth_tokens)
        return self.oauth_tokens

    def request_oauth_tokens_by_refresh_token(self, refresh_token):
        """ Request new OAuth tokens using a refresh token.

        :param refresh_token: The refresh token used to request new tokens.
        :type refresh_token: str
        :return: OAuth tokens
        :rtype: OAuthTokens
        """
        self._oauth_tokens = _UriOAuthService.get_access_token(
            client_id=self.client_id,
            client_secret=self.client_secret,
            grant_type='refresh_token',
            refresh_token=refresh_token,
            environment=self.environment,
            scope=_UriOAuthService.SCOPE[(self.environment, self._oauth_scope)],
            oauth_scope=self._oauth_scope,
            tenant=self.tenant
        )
        if self.token_refreshed_callback is not None:
            self.token_refreshed_callback(self.oauth_tokens)
        return self.oauth_tokens

    @property
    def client_secret(self):
        """ The client secret corresponding to your registered application, or None if your app is a desktop or mobile app.

        :rtype: str
        """

        return self._client_secret

    @property
    def redirection_uri(self):
        """ The URI to which your client browser will be redirected after receiving user consent.

        :rtype: str
        """

        return self._redirection_uri

    @property
    def token_refreshed_callback(self):
        """ The callback function registered, will be invoked after oauth tokens has been refreshed.

        :rtype: OAuthTokens->None
        """

        return self._token_refreshed_callback

    @client_secret.setter
    def client_secret(self, client_secret):
        self._client_secret = client_secret

    @redirection_uri.setter
    def redirection_uri(self, redirection_uri):
        self._redirection_uri = redirection_uri

    @token_refreshed_callback.setter
    def token_refreshed_callback(self, token_refreshed_callback):
        self._token_refreshed_callback = token_refreshed_callback


class OAuthDesktopMobileAuthCodeGrant(OAuthWithAuthorizationCode):
    """ Represents an OAuth authorization object implementing the authorization code grant flow for use in a desktop
    or mobile application.

    You can use an instance of this class as the AuthorizationData.Authentication property
    to authenticate with Bing Ads REST API services.
    """

    def __init__(self, client_id, oauth_tokens=None, env=PRODUCTION, oauth_scope=MSADS_MANAGE, tenant='common', use_msa_prod=True):
        """ Initializes a new instance of this class with the specified client id.

        :param client_id: The client identifier corresponding to your registered application.
        :type client_id: str
        :param oauth_tokens: OAuth token information
        :type oauth_tokens: OAuthTokens
        """
        effective_scope = MSA_PROD if env == SANDBOX and use_msa_prod else oauth_scope
        super(OAuthDesktopMobileAuthCodeGrant, self).__init__(
            client_id,
            None,  # client secret not needed for desktop/mobile apps
            _UriOAuthService.REDIRECTION_URI[(env, effective_scope)],
            oauth_tokens=oauth_tokens,
            env=env,
            oauth_scope=effective_scope,
            tenant=tenant,
            use_msa_prod=use_msa_prod
        )


class OAuthWebAuthCodeGrant(OAuthWithAuthorizationCode):
    """ Represents an OAuth authorization object implementing the authorization code grant flow for use in a web application.

    You can use an instance of this class as the AuthorizationData.Authentication property
    to authenticate with Bing Ads REST API services.
    """
    pass


class OAuthDesktopMobileImplicitGrant(OAuthAuthorization):
    """ Represents an OAuth authorization object implementing the implicit grant flow for use in a desktop or mobile application.

    You can use an instance of this class as the AuthorizationData.Authentication property
    to authenticate with Bing Ads REST API services.
    """

    def __init__(self, client_id, oauth_tokens=None, env=PRODUCTION, oauth_scope=MSADS_MANAGE, tenant='common',
                 use_msa_prod=True):
        """ Initialize an instance of this class.

        :param client_id: The client identifier corresponding to your registered application.
        :type client_id: str
        :param oauth_tokens: OAuth token information
        :type oauth_tokens: OAuthTokens
        """
        effective_scope = MSA_PROD if env == SANDBOX and use_msa_prod else oauth_scope
        super(OAuthDesktopMobileImplicitGrant, self).__init__(client_id, oauth_tokens=oauth_tokens, env=env,
                                                              oauth_scope=effective_scope, tenant=tenant)

    def get_authorization_endpoint(self):
        """ Gets the Microsoft Account authorization endpoint for implicit grant flow.

        :return: The Microsoft Account authorization endpoint.
        :rtype: str
        """
        endpoint_url = _UriOAuthService.AUTHORIZE_URI[(self.environment, self._oauth_scope)]
        if self.environment == PRODUCTION and (self._oauth_scope == MSADS_MANAGE or self._oauth_scope == ADS_MANAGE):
            endpoint_url = endpoint_url.replace('common', self.tenant)

        endpoint = str.format(
            endpoint_url,
            self.client_id,
            'token',
            _UriOAuthService.REDIRECTION_URI[(self.environment, self._oauth_scope)],
        )

        return endpoint if self.state is None else endpoint + '&state=' + self.state

    def extract_access_token_from_uri(self, redirection_uri):
        """ Extract the access token from the specified redirect URI.

        :param redirection_uri: The redirect URI that contains an access token
        :type redirection_uri: str
        :return: OAuth tokens
        :rtype: OAuthTokens
        """
        parameters = parse_qs(urlparse(redirection_uri).fragment)
        if 'access_token' not in parameters or len(parameters['access_token']) == 0:
            raise ValueError(f"Input URI: {redirection_uri} doesn't contain access_token parameter")
        access_token = parameters['access_token'][0]
        if 'expires_in' not in parameters or len(parameters['expires_in']) == 0:
            expires_in = None
        else:
            expires_in = parameters['expires_in'][0]
        self._oauth_tokens = OAuthTokens(
            access_token,
            int(expires_in) if expires_in is not None else None
        )
        return self.oauth_tokens

    @property
    def redirection_uri(self):
        return _UriOAuthService.REDIRECTION_URI[(self.environment, self._oauth_scope)]


class _UriOAuthService:
    """ Provides method for getting OAuth tokens from the live.com authorization server."""

    def __init__(self):
        pass

    REDIRECTION_URI={
        (PRODUCTION, MSADS_MANAGE):   'https://login.microsoftonline.com/common/oauth2/nativeclient',
        (PRODUCTION, ADS_MANAGE):     'https://login.microsoftonline.com/common/oauth2/nativeclient',
        (PRODUCTION, BINGADS_MANAGE): 'https://login.live.com/oauth20_desktop.srf',
        (SANDBOX, MSADS_MANAGE):      'https://login.windows-ppe.net/common/oauth2/nativeclient',
        (SANDBOX, MSA_PROD):          'https://login.microsoftonline.com/common/oauth2/nativeclient'
    }
    AUTH_TOKEN_URI={
        (PRODUCTION, MSADS_MANAGE):   'https://login.microsoftonline.com/common/oauth2/v2.0/token',
        (PRODUCTION, ADS_MANAGE):     'https://login.microsoftonline.com/common/oauth2/v2.0/token',
        (PRODUCTION, BINGADS_MANAGE): 'https://login.live.com/oauth20_token.srf',
        (SANDBOX, MSADS_MANAGE):      'https://login.windows-ppe.net/consumers/oauth2/v2.0/token',
        (SANDBOX, MSA_PROD):          'https://login.microsoftonline.com/common/oauth2/v2.0/token'
    }
    AUTHORIZE_URI={
        (PRODUCTION, MSADS_MANAGE):   'https://login.microsoftonline.com/common/oauth2/v2.0/authorize?client_id={0}&scope=https%3A%2F%2Fads.microsoft.com%2Fmsads.manage%20offline_access&response_type={1}&redirect_uri={2}',
        (PRODUCTION, ADS_MANAGE):     'https://login.microsoftonline.com/common/oauth2/v2.0/authorize?client_id={0}&scope=https%3A%2F%2Fads.microsoft.com%2Fads.manage%20offline_access&response_type={1}&redirect_uri={2}',
        (PRODUCTION, BINGADS_MANAGE): 'https://login.live.com/oauth20_authorize.srf?client_id={0}&scope=bingads.manage&response_type={1}&redirect_uri={2}',
        (SANDBOX, MSADS_MANAGE):      'https://login.windows-ppe.net/consumers/oauth2/v2.0/authorize?client_id={0}&scope=https://api.ads.microsoft.com/msads.manage%20offline_access&response_type={1}&redirect_uri={2}&prompt=login',
        (SANDBOX, MSA_PROD):          'https://login.microsoftonline.com/common/oauth2/v2.0/authorize?client_id={0}&scope=https%3A%2F%2Fsi.ads.microsoft.com%2Fmsads.manage%20offline_access&response_type={1}&redirect_uri={2}'
    }
    SCOPE={
        (PRODUCTION, MSADS_MANAGE):   'https://ads.microsoft.com/msads.manage offline_access',
        (PRODUCTION, ADS_MANAGE):     'https://ads.microsoft.com/ads.manage offline_access',
        (PRODUCTION, BINGADS_MANAGE): 'bingads.manage',
        (SANDBOX, MSADS_MANAGE):      'https://api.ads.microsoft.com/msads.manage offline_access',
        (SANDBOX, MSA_PROD):          'https://si.ads.microsoft.com/msads.manage offline_access'
    }

    @staticmethod
    def get_access_token(**kwargs):

        """ Calls live.com authorization server with parameters passed in, deserializes the response and returns back OAuth tokens.

        :param kwargs: OAuth parameters for authorization server call.
        :return: OAuth tokens.
        :rtype: OAuthTokens
        """

        if 'client_secret' in kwargs and kwargs['client_secret'] is None:
            del kwargs['client_secret']

        if 'oauth_scope' in kwargs and kwargs['oauth_scope'] == 'bingads.manage':
            del kwargs['tenant']

        auth_token_url = _UriOAuthService.AUTH_TOKEN_URI[(kwargs['environment'], kwargs['oauth_scope'])]

        if 'tenant' in kwargs and kwargs['tenant'] is not None:
            auth_token_url = auth_token_url.replace('common', kwargs['tenant'])

        # default timeout set to 300 secs
        r = requests.post(auth_token_url, kwargs, verify=True, timeout=300)
        try:
            r.raise_for_status()
        except Exception:
            error_json = json.loads(r.text)
            raise OAuthTokenRequestException(error_json.get('error'), error_json.get('error_description'))

        r_json = json.loads(r.text)
        return OAuthTokens(r_json['access_token'], int(r_json['expires_in']), r_json['refresh_token'], r_json)



from bingads.v13.bulk.entities import *
from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.extensions import *
from .bulk_audience import BulkAudience

class BulkCustomerList(BulkAudience):
    """ Represents a CustomerList that can be read or written in a bulk file.

    This class exposes the :attr:`customer_list` property that can be read and written as fields of the
    CustomerList record in a bulk file.

    For more information, see CustomerList at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 customer_list=None,
                 status=None,
                 action_type = None):
        super(BulkCustomerList, self).__init__(audience = customer_list, status = status)
        self._action_type = action_type


    _MAPPINGS = [
        _SimpleBulkMapping(
            _StringTable.ActionType,
            field_to_csv=lambda c: bulk_str(c.action_type),
            csv_to_field=lambda c, v: setattr(c, 'action_type', v)
        )
    ]
    
    @property
    def action_type(self):
        """ Defines a action_type """

        return self._action_type

    @action_type.setter
    def action_type(self, action_type):
        self._action_type = action_type

    @property
    def customer_list(self):
        """ Defines a CustomerList """

        return self._audience

    @customer_list.setter
    def customer_list(self, customer_list):
        self._audience = customer_list

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.customer_list, 'customer_list')
        super(BulkCustomerList, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkCustomerList._MAPPINGS)

    def process_mappings_from_row_values(self, row_values):
        self.customer_list = _CAMPAIGN_OBJECT_FACTORY_V13.create('Audience')
        super(BulkCustomerList, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkCustomerList._MAPPINGS)




# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from typing_extensions import Self

class ClipchampTemplateInfo(BaseModel):
    """
    ClipchampTemplateInfo
    """ # noqa: E501
    template_id: Optional[StrictStr] = Field(default=None, alias="TemplateId")
    template_name: Optional[StrictStr] = Field(default=None, alias="TemplateName")
    template_description: Optional[StrictStr] = Field(default=None, alias="TemplateDescription")
    duration: Optional[StrictInt] = Field(default=None, alias="Duration")
    aspect_ratio: Optional[StrictStr] = Field(default=None, alias="AspectRatio")
    themes: Optional[List[StrictStr]] = Field(default=None, alias="Themes")
    number_of_images: Optional[StrictInt] = Field(default=None, alias="NumberOfImages")
    number_of_text: Optional[StrictInt] = Field(default=None, alias="NumberOfText")
    number_of_logos: Optional[StrictInt] = Field(default=None, alias="NumberOfLogos")
    template_thumbnail_url: Optional[StrictStr] = Field(default=None, alias="TemplateThumbnailUrl")
    template_preview_url: Optional[StrictStr] = Field(default=None, alias="TemplatePreviewUrl")
    __properties: ClassVar[List[str]] = ["TemplateId", "TemplateName", "TemplateDescription", "Duration", "AspectRatio", "Themes", "NumberOfImages", "NumberOfText", "NumberOfLogos", "TemplateThumbnailUrl", "TemplatePreviewUrl"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if template_id (nullable) is None
        # and model_fields_set contains the field
        if self.template_id is None and "template_id" in self.model_fields_set:
            _dict['TemplateId'] = None

        # set to None if template_name (nullable) is None
        # and model_fields_set contains the field
        if self.template_name is None and "template_name" in self.model_fields_set:
            _dict['TemplateName'] = None

        # set to None if template_description (nullable) is None
        # and model_fields_set contains the field
        if self.template_description is None and "template_description" in self.model_fields_set:
            _dict['TemplateDescription'] = None

        # set to None if duration (nullable) is None
        # and model_fields_set contains the field
        if self.duration is None and "duration" in self.model_fields_set:
            _dict['Duration'] = None

        # set to None if aspect_ratio (nullable) is None
        # and model_fields_set contains the field
        if self.aspect_ratio is None and "aspect_ratio" in self.model_fields_set:
            _dict['AspectRatio'] = None

        # set to None if themes (nullable) is None
        # and model_fields_set contains the field
        if self.themes is None and "themes" in self.model_fields_set:
            _dict['Themes'] = None

        # set to None if number_of_images (nullable) is None
        # and model_fields_set contains the field
        if self.number_of_images is None and "number_of_images" in self.model_fields_set:
            _dict['NumberOfImages'] = None

        # set to None if number_of_text (nullable) is None
        # and model_fields_set contains the field
        if self.number_of_text is None and "number_of_text" in self.model_fields_set:
            _dict['NumberOfText'] = None

        # set to None if number_of_logos (nullable) is None
        # and model_fields_set contains the field
        if self.number_of_logos is None and "number_of_logos" in self.model_fields_set:
            _dict['NumberOfLogos'] = None

        # set to None if template_thumbnail_url (nullable) is None
        # and model_fields_set contains the field
        if self.template_thumbnail_url is None and "template_thumbnail_url" in self.model_fields_set:
            _dict['TemplateThumbnailUrl'] = None

        # set to None if template_preview_url (nullable) is None
        # and model_fields_set contains the field
        if self.template_preview_url is None and "template_preview_url" in self.model_fields_set:
            _dict['TemplatePreviewUrl'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ClipchampTemplateInfo from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "TemplateId": obj.get("TemplateId") if obj.get("TemplateId") is not None else None,
                        "TemplateName": obj.get("TemplateName") if obj.get("TemplateName") is not None else None,
                        "TemplateDescription": obj.get("TemplateDescription") if obj.get("TemplateDescription") is not None else None,
                        "Duration": obj.get("Duration") if obj.get("Duration") is not None else None,
                        "AspectRatio": obj.get("AspectRatio") if obj.get("AspectRatio") is not None else None,
                        "Themes": obj.get("Themes"),
                        "NumberOfImages": obj.get("NumberOfImages") if obj.get("NumberOfImages") is not None else None,
                        "NumberOfText": obj.get("NumberOfText") if obj.get("NumberOfText") is not None else None,
                        "NumberOfLogos": obj.get("NumberOfLogos") if obj.get("NumberOfLogos") is not None else None,
                        "TemplateThumbnailUrl": obj.get("TemplateThumbnailUrl") if obj.get("TemplateThumbnailUrl") is not None else None,
                        "TemplatePreviewUrl": obj.get("TemplatePreviewUrl") if obj.get("TemplatePreviewUrl") is not None else None
        })
        return _obj



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class ActionAdExtensionActionType(str, Enum):
    UNKNOWN = 'Unknown'

    ACTNOW = 'ActNow'

    APPLYNOW = 'ApplyNow'

    BETNOW = 'BetNow'

    BIDNOW = 'BidNow'

    BOOKACAR = 'BookACar'

    BOOKHOTEL = 'BookHotel'

    BOOKNOW = 'BookNow'

    BROWSE = 'Browse'

    BUYNOW = 'BuyNow'

    CHATNOW = 'ChatNow'

    COMPARE = 'Compare'

    CONTACTUS = 'ContactUs'

    COUPON = 'Coupon'

    DONATE = 'Donate'

    DOWNLOAD = 'Download'

    EMAILNOW = 'EmailNow'

    ENROLLNOW = 'EnrollNow'

    EXPLORE = 'Explore'

    FILENOW = 'FileNow'

    FINDJOB = 'FindJob'

    FREEPLAY = 'FreePlay'

    FREEQUOTE = 'FreeQuote'

    FREETRIAL = 'FreeTrial'

    GETDEALS = 'GetDeals'

    GETOFFER = 'GetOffer'

    GETQUOTE = 'GetQuote'

    JOINNOW = 'JoinNow'

    LEARNMORE = 'LearnMore'

    LISTENNOW = 'ListenNow'

    LOGIN = 'LogIn'

    MESSAGE = 'Message'

    NEWCARS = 'NewCars'

    ORDERNOW = 'OrderNow'

    PLAYGAME = 'PlayGame'

    PLAYNOW = 'PlayNow'

    POSTJOB = 'PostJob'

    REGISTER = 'Register'

    RENTACAR = 'RentACar'

    RENTNOW = 'RentNow'

    RESERVE = 'Reserve'

    SALE = 'Sale'

    SAVENOW = 'SaveNow'

    SCHEDULE = 'Schedule'

    SEEMENU = 'SeeMenu'

    SEEMORE = 'SeeMore'

    SEEOFFER = 'SeeOffer'

    SELLNOW = 'SellNow'

    SHOPNOW = 'ShopNow'

    SHOWTIMES = 'Showtimes'

    SIGNIN = 'SignIn'

    SIGNUP = 'SignUp'

    STARTFREE = 'StartFree'

    STARTNOW = 'StartNow'

    SUBSCRIBE = 'Subscribe'

    TESTDRIVE = 'TestDrive'

    TRYNOW = 'TryNow'

    USEDCARS = 'UsedCars'

    VIEWCARS = 'ViewCars'

    VIEWNOW = 'ViewNow'

    VIEWPLANS = 'ViewPlans'

    VISITSITE = 'VisitSite'

    VOTENOW = 'VoteNow'

    WATCH = 'Watch'

    WATCHMORE = 'WatchMore'

    WATCHNOW = 'WatchNow'

    DIRECTIONS = 'Directions'

    FINDSTORE = 'FindStore'

    SWITCHNOW = 'SwitchNow'

    VISITSTORE = 'VisitStore'

    RENEWNOW = 'RenewNow'

    REORDER = 'Reorder'

    BUILDNOW = 'BuildNow'

    DEALERS = 'Dealers'

    GETDEMO = 'GetDemo'

    GETNOW = 'GetNow'

    GOTODEMO = 'GoToDemo'

    SEEDEMO = 'SeeDemo'

    SEEMODELS = 'SeeModels'

    SEEOFFERS = 'SeeOffers'

    VIEWDEMO = 'ViewDemo'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ActionAdExtensionActionType from a JSON string"""
        return cls(json.loads(json_str))





from .bulk_object import _BulkObject
from .format_version import _FormatVersion
from .object_reader import _BulkObjectReader
from bingads.internal.error_messages import _ErrorMessages


class _BulkRecordReader(object):
    """ Reads a bulk object and also its related data (for example corresponding errors) from the stream."""

    __SUPPORTED_VERSIONS = ["6", "6.0"]

    def __init__(self, object_reader):

        self._passed_first_row = False
        self._bulk_object_reader = object_reader
        self._bulk_object_reader.__enter__()
        self._next_object = None

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._bulk_object_reader.__exit__(exc_type, exc_val, exc_tb)

    def __iter__(self):
        return self

    def __next__(self):
        return self.read()

    def close(self):
        self.__exit__(None, None, None)

    def next(self):
        return self.__next__()

    def read(self):
        """ Returns the next object from the file

        :return: next object
        :rtype: :class:`._BulkObject`
        """

        _, bulk_object = self.try_read(_BulkObject)
        return bulk_object

    def try_read(self, return_type, predicate=lambda x: True):
        """ Reads the object only if it has a certain type

        :param return_type: The type of object that should be returned
        :type return_type: class
        :param predicate: A test that should be run against the object
        :type predicate: function accepting on argument of the BulkObject
        :return: an object of the type specified
        :rtype: (bool, _BulkObject)
        """
        peeked = self._peek()
        if peeked is not None and isinstance(peeked, return_type) and predicate(peeked):
            self._next_object = None
            peeked.read_related_data_from_stream(self)
            return True, peeked
        return False, None

    def _peek(self):
        if not self._passed_first_row:
            first_row_object = self._bulk_object_reader.read_next_bulk_object()
            if isinstance(first_row_object, _FormatVersion):
                if first_row_object.value not in _BulkRecordReader.__SUPPORTED_VERSIONS:
                    raise NotImplementedError(
                        _ErrorMessages.get_format_version_not_supported_message(str(first_row_object.value)))
            else:
                self._next_object = first_row_object
            self._passed_first_row = True
        if self._next_object is not None:
            return self._next_object
        self._next_object = self._bulk_object_reader.read_next_bulk_object()
        return self._next_object




from bingads.v13.bulk.entities import QualityScoreData
from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13

from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping, _ComplexBulkMapping
from bingads.v13.internal.extensions import *


class BulkExperiment(_SingleRecordBulkEntity):
    """ Represents an experiment.

    This class exposes the property :attr:`experiment` that can be read and written as fields of the Experiment record
    in a bulk file.

    For more information, see Experiment at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, experiment=None):
        super(BulkExperiment, self).__init__()
        self._experiment = experiment

    @property
    def experiment(self):
        """ The experiment.
        """
        return self._experiment

    @experiment.setter
    def experiment(self, experiment):
        self._experiment = experiment

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.experiment.Id),
            csv_to_field=lambda c, v: setattr(c.experiment, 'Id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: c.experiment.ExperimentStatus,
            csv_to_field=lambda c, v: setattr(c.experiment, 'ExperimentStatus', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Name,
            field_to_csv=lambda c: c.experiment.Name,
            csv_to_field=lambda c, v: setattr(c.experiment, 'Name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.StartDate,
            field_to_csv=lambda c: bulk_date_str(c.experiment.StartDate),
            csv_to_field=lambda c, v: setattr(c.experiment, 'StartDate', parse_date(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.EndDate,
            field_to_csv=lambda c: field_to_csv_SchedulingDate(c.experiment.EndDate, c.experiment.Id),
            csv_to_field=lambda c, v: setattr(c.experiment, 'EndDate', parse_date(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.TrafficSplitPercent,
            field_to_csv=lambda c: bulk_str(c.experiment.TrafficSplitPercent),
            csv_to_field=lambda c, v: setattr(c.experiment, 'TrafficSplitPercent', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.BaseCampaignId,
            field_to_csv=lambda c: bulk_str(c.experiment.BaseCampaignId),
            csv_to_field=lambda c, v: setattr(c.experiment, 'BaseCampaignId', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ExperimentCampaignId,
            field_to_csv=lambda c: bulk_str(c.experiment.ExperimentCampaignId),
            csv_to_field=lambda c, v: setattr(c.experiment, 'ExperimentCampaignId', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ExperimentType,
            field_to_csv=lambda c: c.experiment.ExperimentType,
            csv_to_field=lambda c, v: setattr(c.experiment, 'ExperimentType', v)
        ),
    ]

    def process_mappings_from_row_values(self, row_values):
        self.experiment = _CAMPAIGN_OBJECT_FACTORY_V13.create('Experiment')
        row_values.convert_to_entity(self, BulkExperiment._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self._experiment, 'Experiment')
        self.convert_to_values(row_values, BulkExperiment._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkExperiment, self).read_additional_data(stream_reader)


# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.campaign.ad_recommendation_customized_property import AdRecommendationCustomizedProperty
from openapi_client.models.campaign.ad_recommendation_image_asset_property import AdRecommendationImageAssetProperty
from openapi_client.models.campaign.ad_recommendation_text_asset_property import AdRecommendationTextAssetProperty
from typing_extensions import Self

class AdRecommendationImageSuggestionMetadata(BaseModel):
    """
    AdRecommendationImageSuggestionMetadata
    """ # noqa: E501
    template_id: Optional[StrictStr] = Field(default=None, alias="TemplateId")
    template_version: Optional[StrictStr] = Field(default=None, alias="TemplateVersion")
    text_asset_properties: Optional[List[Optional[AdRecommendationTextAssetProperty]]] = Field(default=None, alias="TextAssetProperties")
    image_asset_properties: Optional[List[Optional[AdRecommendationImageAssetProperty]]] = Field(default=None, alias="ImageAssetProperties")
    customized_properties: Optional[List[Optional[AdRecommendationCustomizedProperty]]] = Field(default=None, alias="CustomizedProperties")
    __properties: ClassVar[List[str]] = ["TemplateId", "TemplateVersion", "TextAssetProperties", "ImageAssetProperties", "CustomizedProperties"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in text_asset_properties (list)
        _items = []
        if self.text_asset_properties:
            for _item_text_asset_properties in self.text_asset_properties:
                if _item_text_asset_properties:
                    _items.append(_item_text_asset_properties.to_dict())
            _dict['TextAssetProperties'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in image_asset_properties (list)
        _items = []
        if self.image_asset_properties:
            for _item_image_asset_properties in self.image_asset_properties:
                if _item_image_asset_properties:
                    _items.append(_item_image_asset_properties.to_dict())
            _dict['ImageAssetProperties'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in customized_properties (list)
        _items = []
        if self.customized_properties:
            for _item_customized_properties in self.customized_properties:
                if _item_customized_properties:
                    _items.append(_item_customized_properties.to_dict())
            _dict['CustomizedProperties'] = _items
        # set to None if template_id (nullable) is None
        # and model_fields_set contains the field
        if self.template_id is None and "template_id" in self.model_fields_set:
            _dict['TemplateId'] = None

        # set to None if template_version (nullable) is None
        # and model_fields_set contains the field
        if self.template_version is None and "template_version" in self.model_fields_set:
            _dict['TemplateVersion'] = None

        # set to None if text_asset_properties (nullable) is None
        # and model_fields_set contains the field
        if self.text_asset_properties is None and "text_asset_properties" in self.model_fields_set:
            _dict['TextAssetProperties'] = None

        # set to None if image_asset_properties (nullable) is None
        # and model_fields_set contains the field
        if self.image_asset_properties is None and "image_asset_properties" in self.model_fields_set:
            _dict['ImageAssetProperties'] = None

        # set to None if customized_properties (nullable) is None
        # and model_fields_set contains the field
        if self.customized_properties is None and "customized_properties" in self.model_fields_set:
            _dict['CustomizedProperties'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AdRecommendationImageSuggestionMetadata from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "TemplateId": obj.get("TemplateId") if obj.get("TemplateId") is not None else None,
                        "TemplateVersion": obj.get("TemplateVersion") if obj.get("TemplateVersion") is not None else None,
                        "TextAssetProperties": [AdRecommendationTextAssetProperty.from_dict(_item) for _item in obj["TextAssetProperties"]] if obj.get("TextAssetProperties") is not None else None,
                        "ImageAssetProperties": [AdRecommendationImageAssetProperty.from_dict(_item) for _item in obj["ImageAssetProperties"]] if obj.get("ImageAssetProperties") is not None else None,
                        "CustomizedProperties": [AdRecommendationCustomizedProperty.from_dict(_item) for _item in obj["CustomizedProperties"]] if obj.get("CustomizedProperties") is not None else None
        })
        return _obj



class ReportingOperationStatus(object):
    """ Contains tracking details about the Report Request Status """

    def __init__(self,
                 status=None,
                 report_download_url=None):
        """ Initialize a new instance of this class.

        :param status: (optional) The download or upload status.
        :type status: str
        :param report_download_url: (optional) The report download Url.
        :type report_download_url: str
        """

        self._status = status
        self._report_download_url = report_download_url

    @property
    def status(self):
        """ The download status.
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, value):
        self._status = value


    @property
    def report_download_url(self):
        """ The report download Url.
        :rtype: str
        """
        return self._report_download_url

    @report_download_url.setter
    def report_download_url(self, value):
        self._report_download_url = value



# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class PerformanceInsightsMessageTemplateId(str, Enum):
    FLUXIMPRINCREASEACCT = 'FluxImprIncreaseAcct'

    FLUXIMPRDECREASEACCT = 'FluxImprDecreaseAcct'

    FLUXIMPRINCREASEINFINITYACCT = 'FluxImprIncreaseInfinityAcct'

    FLUXIMPRINCREASEACCTPREVDAY = 'FluxImprIncreaseAcctPrevDay'

    FLUXIMPRDECREASEACCTPREVDAY = 'FluxImprDecreaseAcctPrevDay'

    FLUXIMPRINCREASEINFINITYACCTPREVDAY = 'FluxImprIncreaseInfinityAcctPrevDay'

    FLUXCLICKINCREASEACCT = 'FluxClickIncreaseAcct'

    FLUXCLICKDECREASEACCT = 'FluxClickDecreaseAcct'

    FLUXCLICKINCREASEINFINITYACCT = 'FluxClickIncreaseInfinityAcct'

    FLUXCLICKINCREASEACCTPREVDAY = 'FluxClickIncreaseAcctPrevDay'

    FLUXCLICKDECREASEACCTPREVDAY = 'FluxClickDecreaseAcctPrevDay'

    FLUXCLICKINCREASEINFINITYACCTPREVDAY = 'FluxClickIncreaseInfinityAcctPrevDay'

    FLUXCOSTINCREASEACCT = 'FluxCostIncreaseAcct'

    FLUXCOSTDECREASEACCT = 'FluxCostDecreaseAcct'

    FLUXCOSTINCREASEINFINITYACCT = 'FluxCostIncreaseInfinityAcct'

    FLUXCOSTINCREASEACCTPREVDAY = 'FluxCostIncreaseAcctPrevDay'

    FLUXCOSTDECREASEACCTPREVDAY = 'FluxCostDecreaseAcctPrevDay'

    FLUXCOSTINCREASEINFINITYACCTPREVDAY = 'FluxCostIncreaseInfinityAcctPrevDay'

    FLUXIMPRINCREASECAMPAIGN = 'FluxImprIncreaseCampaign'

    FLUXIMPRDECREASECAMPAIGN = 'FluxImprDecreaseCampaign'

    FLUXIMPRINCREASEINFINITYCAMPAIGN = 'FluxImprIncreaseInfinityCampaign'

    FLUXIMPRINCREASECAMPAIGNPREVDAY = 'FluxImprIncreaseCampaignPrevDay'

    FLUXIMPRDECREASECAMPAIGNPREVDAY = 'FluxImprDecreaseCampaignPrevDay'

    FLUXIMPRINCREASEINFINITYCAMPAIGNPREVDAY = 'FluxImprIncreaseInfinityCampaignPrevDay'

    FLUXCLICKINCREASECAMPAIGN = 'FluxClickIncreaseCampaign'

    FLUXCLICKDECREASECAMPAIGN = 'FluxClickDecreaseCampaign'

    FLUXCLICKINCREASEINFINITYCAMPAIGN = 'FluxClickIncreaseInfinityCampaign'

    FLUXCLICKINCREASECAMPAIGNPREVDAY = 'FluxClickIncreaseCampaignPrevDay'

    FLUXCLICKDECREASECAMPAIGNPREVDAY = 'FluxClickDecreaseCampaignPrevDay'

    FLUXCLICKINCREASEINFINITYCAMPAIGNPREVDAY = 'FluxClickIncreaseInfinityCampaignPrevDay'

    FLUXCOSTINCREASECAMPAIGN = 'FluxCostIncreaseCampaign'

    FLUXCOSTDECREASECAMPAIGN = 'FluxCostDecreaseCampaign'

    FLUXCOSTINCREASEINFINITYCAMPAIGN = 'FluxCostIncreaseInfinityCampaign'

    FLUXCOSTINCREASECAMPAIGNPREVDAY = 'FluxCostIncreaseCampaignPrevDay'

    FLUXCOSTDECREASECAMPAIGNPREVDAY = 'FluxCostDecreaseCampaignPrevDay'

    FLUXCOSTINCREASEINFINITYCAMPAIGNPREVDAY = 'FluxCostIncreaseInfinityCampaignPrevDay'

    FLUXCONVERSIONDECREASEACCT = 'FluxConversionDecreaseAcct'

    FLUXCONVERSIONZEROACCT = 'FluxConversionZeroAcct'

    RCAUDIENCEINCREASE = 'RCAudienceIncrease'

    RCAUDIENCEDECREASE = 'RCAudienceDecrease'

    RCBIDINCREASE = 'RCBidIncrease'

    RCBIDDECREASE = 'RCBidDecrease'

    RCBUDGETINCREASE = 'RCBudgetIncrease'

    RCBUDGETDECREASE = 'RCBudgetDecrease'

    RCCLICKINCREASE = 'RCClickIncrease'

    RCCLICKDECREASE = 'RCClickDecrease'

    RCCONVERSIONGOALISSUE = 'RCConversionGoalIssue'

    RCDEVICETARGETINGBIDCHANGE = 'RCDeviceTargetingBidChange'

    RCEDITORIALAPPROVAL = 'RCEditorialApproval'

    RCEDITORIALDISAPPROVAL = 'RCEditorialDisapproval'

    RCENTITYADDITIONWITHCOUNT = 'RCEntityAdditionWithCount'

    RCENTITYDELETIONWITHCOUNT = 'RCEntityDeletionWithCount'

    RCENTITYPAUSEWITHCOUNT = 'RCEntityPauseWithCount'

    RCENTITYUNPAUSEWITHCOUNT = 'RCEntityUnpauseWithCount'

    RCMORECOMPETITIVE = 'RCMoreCompetitive'

    RCNETWORKLESSRESTRICT = 'RCNetworkLessRestrict'

    RCNETWORKMORERESTRICT = 'RCNetworkMoreRestrict'

    RCPAYMENTISSUEINVOICEACCOUNTPAUSED = 'RCPaymentIssueInvoiceAccountPaused'

    RCPAYMENTISSUEINVOICEACCOUNTONHOLD = 'RCPaymentIssueInvoiceAccountOnHold'

    RCPAYMENTISSUEPREPAYACCOUNTPAUSED = 'RCPaymentIssuePrepayAccountPaused'

    RCPAYMENTISSUETHRESHOLDACCOUNTONHOLD = 'RCPaymentIssueThresholdAccountOnHold'

    RCPAYMENTISSUEYESTERDAY = 'RCPaymentIssueYesterday'

    RCSEARCHTERMINCREASE = 'RCSearchTermIncrease'

    RCSEARCHTERMINCREASECLICK = 'RCSearchTermIncreaseClick'

    RCSEARCHTERMDECREASE = 'RCSearchTermDecrease'

    RCSEARCHTERMDECREASECLICK = 'RCSearchTermDecreaseClick'

    RCSEARCHTERMCOMPETITIONINCREASE = 'RCSearchTermCompetitionIncrease'

    RCSEARCHTERMCOMPETITIONINCREASECLICK = 'RCSearchTermCompetitionIncreaseClick'

    RCSEARCHTERMCOMPETITIONDECREASE = 'RCSearchTermCompetitionDecrease'

    RCSEARCHTERMCOMPETITIONDECREASECLICK = 'RCSearchTermCompetitionDecreaseClick'

    RCSINGLECAMPAIGNADDITION = 'RCSingleCampaignAddition'

    RCSINGLECAMPAIGNPAUSE = 'RCSingleCampaignPause'

    RCSINGLECAMPAIGNUNPAUSE = 'RCSingleCampaignUnpause'

    RCSINGLECAMPAIGNBUDGETINCREASE = 'RCSingleCampaignBudgetIncrease'

    RCSINGLECAMPAIGNBUDGETDECREASE = 'RCSingleCampaignBudgetDecrease'

    RCSINGLECAMPAIGNDEVICETARGETINGBIDCHANGE = 'RCSingleCampaignDeviceTargetingBidChange'

    RCSTRONGERCOMPETITORADQUALITY = 'RCStrongerCompetitorAdQuality'

    RCSTRONGERCOMPETITORBID = 'RCStrongerCompetitorBid'

    RCSYNDTRAFFICINCREASE = 'RCSyndTrafficIncrease'

    RCSYNDTRAFFICDECREASE = 'RCSyndTrafficDecrease'

    ACTADSOPPR = 'ActAdsOppr'

    ACTALLAVAILABLEOPPR = 'ActAllAvailableOppr'

    ACTBMADOPTIONOPPR = 'ActBMAdoptionOppr'

    ACTBMMKEYWORDOPPR = 'ActBMMKeywordOppr'

    ACTCAMPAIGNCONTEXTKEYWORDOPPR = 'ActCampaignContextKeywordOppr'

    ACTCHECKPUBLISHERWEBSITE = 'ActCheckPublisherWebsite'

    ACTCOMPETITIONAUCTIONINSIGHT = 'ActCompetitionAuctionInsight'

    ACTCOMPETITIVEBUDGETOPPR = 'ActCompetitiveBudgetOppr'

    ACTCOMPETITIVEKEYWORDOPPR = 'ActCompetitiveKeywordOppr'

    ACTCOMPETITIVELOCATIONTARGETOPPR = 'ActCompetitiveLocationTargetOppr'

    ACTCTRQUALITYBUNDLE = 'ActCTRQualityBundle'

    ACTEDITORIALREVIEWPROCESS = 'ActEditorialReviewProcess'

    ACTFIXCONVERSIONGOALSETTINGOPPR = 'ActFixConversionGoalSettingOppr'

    ACTFIXCONVERSIONTRACKINGOPPR = 'ActFixConversionTrackingOppr'

    ACTGENERALBUDGETOPPR = 'ActGeneralBudgetOppr'

    ACTNEWCONVERSIONGOALOPPR = 'ActNewConversionGoalOppr'

    ACTINVOICEACCOUNTPAUSED = 'ActInvoiceAccountPaused'

    ACTINVOICEACCOUNTONHOLD = 'ActInvoiceAccountOnHold'

    ACTPREPAYACCOUNTPAUSED = 'ActPrepayAccountPaused'

    ACTREALLOCATEBUDGETOPPR = 'ActReallocateBudgetOppr'

    ACTREPAIRKEYWORDSOPPR = 'ActRepairKeywordsOppr'

    ACTRSAOPPR = 'ActRSAOppr'

    ACTSEARCHTERM = 'ActSearchTerm'

    ACTSETUPCONVERSIONTRACKINGOPPR = 'ActSetupConversionTrackingOppr'

    ACTSITELINKOPPR = 'ActSiteLinkOppr'

    ACTTHRESHOLDACCOUNTONHOLD = 'ActThresholdAccountOnHold'

    ACTWEBSITEEXCLUSION = 'ActWebsiteExclusion'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of PerformanceInsightsMessageTemplateId from a JSON string"""
        return cls(json.loads(json_str))





from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping, _ComplexBulkMapping
from bingads.v13.internal.extensions import *
from decimal import Decimal


class BulkBidStrategy(_SingleRecordBulkEntity):
    """ Represents a bid strategy that can be read or written in a bulk file.

    Properties of this class and of classes that it is derived from, correspond to fields of the Bid Strategy record in a bulk file.
    For more information, see Bid Strategy at https://go.microsoft.com/fwlink/?linkid=846127

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, bid_strategy=None, status=None, account_id=None):
        super(BulkBidStrategy, self).__init__()
        self._bid_strategy = bid_strategy
        self._status = status
        self._account_id = account_id

    @property
    def bid_strategy(self):
        """
        the Bid Strategy object, see more detail at: https://go.microsoft.com/fwlink/?linkid=846127
        """
        return self._bid_strategy

    @bid_strategy.setter
    def bid_strategy(self, value):
        self._bid_strategy = value

    @property
    def status(self):
        """ the status of bulk record
        Corresponds to the 'Status' field in the bulk file.

        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, value):
        self._status = value

    @property
    def account_id(self):
        """ the id of the account which contains the bid strategy
        Corresponds to the 'Parent Id' field in the bulk file.

        :rtype: long
        """
        return self._account_id

    @account_id.setter
    def account_id(self, value):
        self._account_id = value


    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.bid_strategy.Id),
            csv_to_field=lambda c, v: setattr(c.bid_strategy, 'Id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.account_id),
            csv_to_field=lambda c, v: setattr(c, 'account_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: c.status,
            csv_to_field=lambda c, v: setattr(c, 'status', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.BidStrategyName,
            field_to_csv=lambda c: bulk_str(c.bid_strategy.Name),
            csv_to_field=lambda c, v: setattr(c.bid_strategy, 'Name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.CampaignType,
            field_to_csv=lambda c: bulk_str(c.bid_strategy.AssociatedCampaignType),
            csv_to_field=lambda c, v: setattr(c.bid_strategy, 'AssociatedCampaignType', v if v else None)
        ),
        _ComplexBulkMapping(bid_strategy_biddingscheme_to_csv, csv_to_bid_strategy_biddingscheme),
    ]

    def process_mappings_from_row_values(self, row_values):
        self._bid_strategy = _CAMPAIGN_OBJECT_FACTORY_V13.create('BidStrategy')
        row_values.convert_to_entity(self, BulkBidStrategy._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.bid_strategy, 'bid_strategy')
        self.convert_to_values(row_values, BulkBidStrategy._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkBidStrategy, self).read_additional_data(stream_reader)



class SdkException(Exception):
    """ Base Exception for exceptions in SDK. """

    def __init__(self, message):
        """ Initialize a new instance of this class with detailed message.

        :param message: Detailed exception message.
        :type message: str
        """

        self._message = message

    def __str__(self):
        return self.message

    @property
    def message(self):
        """ Detailed exception message.

        :rtype: str
        """
        return self._message


class OAuthTokenRequestException(SdkException):
    """ This exception is thrown if an error was returned from the Microsoft Account authorization server. """

    def __init__(self, error_code, description):
        """ Initializes a new instance of this class with the specified error code and OAuth error details.

        :param error_code: The error code of the OAuth error.
        :type error_code: str
        :param description: The description of the OAuth error.
        :type description: str
        """

        super(OAuthTokenRequestException, self).__init__(
            str.format("error_code: {0}, error_description: {1}", error_code, description))
        self._error_code = error_code
        self._error_description = description

    @property
    def error_code(self):
        """ The error code of the OAuth error.

        :rtype: str
        """

        return self._error_code

    @property
    def error_description(self):
        """ The description of the OAuth error.

        :rtype: str
        """

        return self._error_description


class FileUploadException(SdkException):
    """ This exception is thrown if timeout occurs """

    def __init__(self, description):
        """ Initializes a new instance of this class with the specified error messages.

        :param description: The description of the file upload error.
        :type description: str
        """
        super(FileUploadException, self).__init__(str(description))


class FileDownloadException(SdkException):
    """ This exception is thrown if timeout occurs """

    def __init__(self, description):
        """ Initializes a new instance of this class with the specified error messages.

        :param description: The description of the file download error.
        :type description: str
        """
        super(FileDownloadException, self).__init__(str(description))


class TimeoutException(SdkException):
    """ This exception is thrown if timeout occurs """

    def __init__(self, description):
        """ Initializes a new instance of this class with the specified error messages.

        :param description: The description of the file download error.
        :type description: str
        """
        super(TimeoutException, self).__init__(str(description))


from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping, _ComplexBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.bulk.entities.common import _ProductConditionHelper
from bingads.v13.internal.extensions import bulk_str


class BulkCampaignProductScope(_SingleRecordBulkEntity):
    """ Represents a Campaign Criterion that can be read or written in a bulk file.

    This class exposes the :attr:`campaign_criterion` property that can be read and written as fields of the
    Campaign Product Scope record in a bulk file.

    For more information, see Campaign Product Scope at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 campaign_name=None,
                 status=None,
                 biddable_campaign_criterion=None):
        super(BulkCampaignProductScope, self).__init__()

        self._campaign_name = campaign_name
        self._status = status
        self._biddable_campaign_criterion = biddable_campaign_criterion

    @classmethod
    def _add_product_condition_to_row_values(cls, entity, value):
        criterion = entity.biddable_campaign_criterion.Criterion
        if criterion is not None and hasattr(criterion, 'Conditions') and criterion.Conditions is not None and \
                hasattr(criterion.Conditions, 'ProductCondition'):
            return _ProductConditionHelper.add_row_values_from_conditions(criterion.Conditions.ProductCondition, value)
        return None

    _MAPPINGS = [
        _SimpleBulkMapping(
            _StringTable.Status,
            field_to_csv=lambda c: c.biddable_campaign_criterion.Status,
            csv_to_field=lambda c, v: setattr(c.biddable_campaign_criterion, 'Status', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.biddable_campaign_criterion.Id),
            csv_to_field=lambda c, v: setattr(c.biddable_campaign_criterion, 'Id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.biddable_campaign_criterion.CampaignId),
            csv_to_field=lambda c, v: setattr(c.biddable_campaign_criterion, 'CampaignId', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Campaign,
            field_to_csv=lambda c: c.campaign_name,
            csv_to_field=lambda c, v: setattr(c, 'campaign_name', v)
        ),
        _ComplexBulkMapping(
            entity_to_csv=lambda c, v: BulkCampaignProductScope._add_product_condition_to_row_values(c, v),
            csv_to_entity=lambda v, c: _ProductConditionHelper.add_conditions_from_row_values(
                v,
                c.biddable_campaign_criterion.Criterion.Conditions.ProductCondition
            )
        )
    ]

    @property
    def campaign_name(self):
        """ The name of the Campaign

        :rtype: str
        """

        return self._campaign_name

    @campaign_name.setter
    def campaign_name(self, campaign_name):
        self._campaign_name = campaign_name

    @property
    def status(self):
        """ The status of the Campaign Criterion

        :rtype: str
        """

        return self._status

    @status.setter
    def status(self, status):
        self._status = status

    @property
    def biddable_campaign_criterion(self):
        """ Defines a Biddable Campaign Criterion """

        return self._biddable_campaign_criterion

    @biddable_campaign_criterion.setter
    def biddable_campaign_criterion(self, biddable_campaign_criterion):
        self._biddable_campaign_criterion = biddable_campaign_criterion

    def process_mappings_from_row_values(self, row_values):
        self._biddable_campaign_criterion = _CAMPAIGN_OBJECT_FACTORY_V13.create('BiddableCampaignCriterion')
        self._biddable_campaign_criterion.Type = 'BiddableCampaignCriterion'
        self._biddable_campaign_criterion.Criterion = _CAMPAIGN_OBJECT_FACTORY_V13.create('ProductScope')
        self._biddable_campaign_criterion.Criterion.Type = 'ProductScope'

        row_values.convert_to_entity(self, BulkCampaignProductScope._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.biddable_campaign_criterion, 'biddable_campaign_criterion')
        self.convert_to_values(row_values, BulkCampaignProductScope._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkCampaignProductScope, self).read_additional_data(stream_reader)



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.campaign.recommendation_asset_property import RecommendationAssetProperty
from openapi_client.models.campaign.recommendation_background import RecommendationBackground
from openapi_client.models.campaign.recommendation_color import RecommendationColor
from openapi_client.models.campaign.recommendation_image import RecommendationImage
from openapi_client.models.campaign.recommendation_text import RecommendationText
from typing_extensions import Self

class RecommendationTemplate(BaseModel):
    """
    RecommendationTemplate
    """ # noqa: E501
    style: Optional[StrictInt] = Field(default=None, alias="style")
    template_id: Optional[StrictStr] = Field(default=None, alias="templateId")
    template_name: Optional[StrictStr] = Field(default=None, alias="templateName")
    version: Optional[StrictStr] = Field(default=None, alias="version")
    image: Optional[RecommendationImage] = Field(default=None, alias="image")
    logo: Optional[RecommendationImage] = Field(default=None, alias="logo")
    business_name: Optional[RecommendationText] = Field(default=None, alias="businessName")
    short_headline: Optional[RecommendationText] = Field(default=None, alias="shortHeadline")
    long_headline: Optional[RecommendationText] = Field(default=None, alias="longHeadline")
    description: Optional[RecommendationText] = Field(default=None, alias="description")
    cta: Optional[RecommendationText] = Field(default=None, alias="cta")
    asset_properties: Optional[List[Optional[RecommendationAssetProperty]]] = Field(default=None, alias="assetProperties")
    background: Optional[RecommendationBackground] = Field(default=None, alias="background")
    color: Optional[List[Optional[RecommendationColor]]] = Field(default=None, alias="color")
    __properties: ClassVar[List[str]] = ["style", "templateId", "templateName", "version", "image", "logo", "businessName", "shortHeadline", "longHeadline", "description", "cta", "assetProperties", "background", "color"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of image
        if self.image:
            _dict['image'] = self.image.to_dict()
        # override the default output from pydantic by calling `to_dict()` of logo
        if self.logo:
            _dict['logo'] = self.logo.to_dict()
        # override the default output from pydantic by calling `to_dict()` of business_name
        if self.business_name:
            _dict['businessName'] = self.business_name.to_dict()
        # override the default output from pydantic by calling `to_dict()` of short_headline
        if self.short_headline:
            _dict['shortHeadline'] = self.short_headline.to_dict()
        # override the default output from pydantic by calling `to_dict()` of long_headline
        if self.long_headline:
            _dict['longHeadline'] = self.long_headline.to_dict()
        # override the default output from pydantic by calling `to_dict()` of description
        if self.description:
            _dict['description'] = self.description.to_dict()
        # override the default output from pydantic by calling `to_dict()` of cta
        if self.cta:
            _dict['cta'] = self.cta.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in asset_properties (list)
        _items = []
        if self.asset_properties:
            for _item_asset_properties in self.asset_properties:
                if _item_asset_properties:
                    _items.append(_item_asset_properties.to_dict())
            _dict['AssetProperties'] = _items
        # override the default output from pydantic by calling `to_dict()` of background
        if self.background:
            _dict['background'] = self.background.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in color (list)
        _items = []
        if self.color:
            for _item_color in self.color:
                if _item_color:
                    _items.append(_item_color.to_dict())
            _dict['Color'] = _items
        # set to None if style (nullable) is None
        # and model_fields_set contains the field
        if self.style is None and "style" in self.model_fields_set:
            _dict['style'] = None

        # set to None if template_id (nullable) is None
        # and model_fields_set contains the field
        if self.template_id is None and "template_id" in self.model_fields_set:
            _dict['templateId'] = None

        # set to None if template_name (nullable) is None
        # and model_fields_set contains the field
        if self.template_name is None and "template_name" in self.model_fields_set:
            _dict['templateName'] = None

        # set to None if version (nullable) is None
        # and model_fields_set contains the field
        if self.version is None and "version" in self.model_fields_set:
            _dict['version'] = None

        # set to None if image (nullable) is None
        # and model_fields_set contains the field
        if self.image is None and "image" in self.model_fields_set:
            _dict['image'] = None

        # set to None if logo (nullable) is None
        # and model_fields_set contains the field
        if self.logo is None and "logo" in self.model_fields_set:
            _dict['logo'] = None

        # set to None if business_name (nullable) is None
        # and model_fields_set contains the field
        if self.business_name is None and "business_name" in self.model_fields_set:
            _dict['businessName'] = None

        # set to None if short_headline (nullable) is None
        # and model_fields_set contains the field
        if self.short_headline is None and "short_headline" in self.model_fields_set:
            _dict['shortHeadline'] = None

        # set to None if long_headline (nullable) is None
        # and model_fields_set contains the field
        if self.long_headline is None and "long_headline" in self.model_fields_set:
            _dict['longHeadline'] = None

        # set to None if description (nullable) is None
        # and model_fields_set contains the field
        if self.description is None and "description" in self.model_fields_set:
            _dict['description'] = None

        # set to None if cta (nullable) is None
        # and model_fields_set contains the field
        if self.cta is None and "cta" in self.model_fields_set:
            _dict['cta'] = None

        # set to None if asset_properties (nullable) is None
        # and model_fields_set contains the field
        if self.asset_properties is None and "asset_properties" in self.model_fields_set:
            _dict['assetProperties'] = None

        # set to None if background (nullable) is None
        # and model_fields_set contains the field
        if self.background is None and "background" in self.model_fields_set:
            _dict['background'] = None

        # set to None if color (nullable) is None
        # and model_fields_set contains the field
        if self.color is None and "color" in self.model_fields_set:
            _dict['color'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of RecommendationTemplate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "style": obj.get("Style") if obj.get("Style") is not None else None,
                        "templateId": obj.get("TemplateId") if obj.get("TemplateId") is not None else None,
                        "templateName": obj.get("TemplateName") if obj.get("TemplateName") is not None else None,
                        "version": obj.get("Version") if obj.get("Version") is not None else None,
                        "image": RecommendationImage.from_dict(obj["Image"]) if obj.get("Image") is not None else None,
                        "logo": RecommendationImage.from_dict(obj["Logo"]) if obj.get("Logo") is not None else None,
                        "businessName": RecommendationText.from_dict(obj["BusinessName"]) if obj.get("BusinessName") is not None else None,
                        "shortHeadline": RecommendationText.from_dict(obj["ShortHeadline"]) if obj.get("ShortHeadline") is not None else None,
                        "longHeadline": RecommendationText.from_dict(obj["LongHeadline"]) if obj.get("LongHeadline") is not None else None,
                        "description": RecommendationText.from_dict(obj["Description"]) if obj.get("Description") is not None else None,
                        "cta": RecommendationText.from_dict(obj["Cta"]) if obj.get("Cta") is not None else None,
                        "assetProperties": [RecommendationAssetProperty.from_dict(_item) for _item in obj["AssetProperties"]] if obj.get("AssetProperties") is not None else None,
                        "background": RecommendationBackground.from_dict(obj["Background"]) if obj.get("Background") is not None else None,
                        "color": [RecommendationColor.from_dict(_item) for _item in obj["Color"]] if obj.get("Color") is not None else None
        })
        return _obj



# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class AdPosition(str, Enum):
    ALL = 'All'

    MAINLINE1 = 'MainLine1'

    MAINLINE2 = 'MainLine2'

    MAINLINE3 = 'MainLine3'

    MAINLINE4 = 'MainLine4'

    FIRSTPAGE1 = 'FirstPage1'

    FIRSTPAGE2 = 'FirstPage2'

    FIRSTPAGE3 = 'FirstPage3'

    FIRSTPAGE4 = 'FirstPage4'

    FIRSTPAGE5 = 'FirstPage5'

    FIRSTPAGE6 = 'FirstPage6'

    FIRSTPAGE7 = 'FirstPage7'

    FIRSTPAGE8 = 'FirstPage8'

    FIRSTPAGE9 = 'FirstPage9'

    FIRSTPAGE10 = 'FirstPage10'

    AGGREGATE = 'Aggregate'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AdPosition from a JSON string"""
        return cls(json.loads(json_str))





from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.extensions import *


class BulkAccount(_SingleRecordBulkEntity):
    """ Represents an account that can be read or written in a bulk file.

    Properties of this class and of classes that it is derived from, correspond to fields of the Account record in a bulk file.
    For more information, see Account at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, account_id=None, customer_id=None, sync_time=None):
        super(BulkAccount, self).__init__()
        self._id = account_id
        self._customer_id = customer_id
        self._sync_time = sync_time
        self._msclkid_auto_tagging_enabled = None
        self._include_view_through_conversions = None
        self._profile_expansion_enabled = None
        self._tracking_url_template = None
        self._final_url_suffix = None
        self._ad_click_parallel_tracking=None
        self._auto_apply_recommendations = None
        self._allow_image_auto_retrieve = None
        self._business_attributes = None

    @property
    def id(self):
        """ The identifier of the account.

        Corresponds to the 'Id' field in the bulk file.

        :return: The identifier of the account.
        :rtype: int
        """

        return self._id

    @property
    def customer_id(self):
        """ The identifier of the customer that contains the account.

        Corresponds to the 'Parent Id' field in the bulk file.

        :return: The identifier of the customer that contains the account.
        :rtype: int
        """

        return self._customer_id

    @property
    def sync_time(self):
        """ The date and time that you last synced your account using the bulk service.

        You should keep track of this value in UTC time.
        Corresponds to the 'Sync Time' field in the bulk file.

        :return: The date and time that you last synced your account using the bulk service.
        :rtype: datetime.datetime
        """

        return self._sync_time

    @property
    def msclkid_auto_tagging_enabled(self):
        """ Determines whether auto-tagging of the MSCLKID query string parameter is enabled. The MSCLKID is a 32-character GUID that is unique for each ad click.
        :return: The msclkid autotag setting of the account
        :rtype: bool
        """
        return self._msclkid_auto_tagging_enabled

    @property
    def include_view_through_conversions(self):
        """
        :return: The 'Include View Through Conversions' field of the account
        :rtype: bool
        """
        return self._include_view_through_conversions

    @property
    def profile_expansion_enabled(self):
        """
        :return: The 'Profile Expansion Enabled' field of the account
        :rtype: bool
        """
        return self._profile_expansion_enabled

    @property
    def tracking_url_template(self):
        """ The tracking template to use as a default for all URLs in your account.

        :return: The tracking template of the account
        :rtype: str
        """
        return self._tracking_url_template

    @property
    def final_url_suffix(self):
        """ The final url suffix to use as a default for all URLs in your account.

        :return: The tracking template of the account
        :rtype: str
        """
        return self._final_url_suffix

    @final_url_suffix.setter
    def final_url_suffix(self, v):
        self._final_url_suffix = v

    @property
    def ad_click_parallel_tracking(self):
        """ The setting of parallel tracking in the account.

        :return: The setting of parallel tracking of the account
        :rtype: bool
        """
        return self._ad_click_parallel_tracking

    @ad_click_parallel_tracking.setter
    def ad_click_parallel_tracking(self, v):
        self._ad_click_parallel_tracking = v

    @property
    def allow_image_auto_retrieve(self):
        """ The setting of allowing image auto retrieve in the account.

        :return: The setting of allowing image auto retrieve of the account
        :rtype: bool
        """
        return self._allow_image_auto_retrieve

    @allow_image_auto_retrieve.setter
    def allow_image_auto_retrieve(self, v):
        self._allow_image_auto_retrieve = v

    @property
    def auto_apply_recommendations(self):
        """ The setting of allowing image auto retrieve in the account.

        :return: The setting of allowing image auto retrieve of the account
        :rtype: dict
        """
        return self._auto_apply_recommendations

    @auto_apply_recommendations.setter
    def auto_apply_recommendations(self, v):
        self._auto_apply_recommendations = v

    @property
    def business_attributes(self):
        """ The setting of business attributes in the account.

        :return: The setting of business attributes of the account
        :rtype: dict
        """
        return self._business_attributes

    @business_attributes.setter
    def business_attributes(self, v):
        self._business_attributes = v

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.id),
            csv_to_field=lambda c, v: setattr(c, '_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.customer_id),
            csv_to_field=lambda c, v: setattr(c, '_customer_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.SyncTime,
            field_to_csv=lambda c: bulk_datetime_str(c.sync_time),
            csv_to_field=lambda c, v: setattr(c, '_sync_time', parse_datetime(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.MSCLKIDAutoTaggingEnabled,
            field_to_csv=lambda c: bulk_str(c.msclkid_auto_tagging_enabled),
            csv_to_field=lambda c, v: setattr(c, '_msclkid_auto_tagging_enabled', parse_bool(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.TrackingTemplate,
            field_to_csv=lambda c: bulk_optional_str(c.tracking_url_template, c.id),
            csv_to_field=lambda c, v: setattr(c, '_tracking_url_template', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FinalUrlSuffix,
            field_to_csv=lambda c: bulk_optional_str(c.final_url_suffix, c.id),
            csv_to_field=lambda c, v: setattr(c, '_final_url_suffix', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.IncludeViewThroughConversions,
            field_to_csv=lambda c: bulk_str(c.include_view_through_conversions),
            csv_to_field=lambda c, v: setattr(c, '_include_view_through_conversions', parse_bool(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.ProfileExpansionEnabled,
            field_to_csv=lambda c: bulk_str(c.profile_expansion_enabled),
            csv_to_field=lambda c, v: setattr(c, '_profile_expansion_enabled', parse_bool(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.AdClickParallelTracking,
            field_to_csv=lambda c: bulk_str(c.ad_click_parallel_tracking),
            csv_to_field=lambda c, v: setattr(c, 'ad_click_parallel_tracking', parse_bool(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.AllowImageAutoRetrieve,
            field_to_csv=lambda c: bulk_str(c.allow_image_auto_retrieve),
            csv_to_field=lambda c, v: setattr(c, 'allow_image_auto_retrieve', parse_bool(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.AutoApplyRecommendations,
            field_to_csv=lambda c: dict_bulk_str(c.auto_apply_recommendations, ';'),
            csv_to_field=lambda c, v: setattr(c, 'auto_apply_recommendations', parse_dict(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.BusinessAttributes,
            field_to_csv=lambda c: multi_bulk_str(c.business_attributes, ';'),
            csv_to_field=lambda c, v: setattr(c, 'business_attributes', parse_multi(v))
        )
    ]

    def process_mappings_from_row_values(self, row_values):
        row_values.convert_to_entity(self, BulkAccount._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self.convert_to_values(row_values, BulkAccount._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkAccount, self).read_additional_data(stream_reader)



# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class AuctionSegment(str, Enum):
    DAY = 'Day'

    WEEK = 'Week'

    MONTH = 'Month'

    QUARTER = 'Quarter'

    DAYOFWEEK = 'DayOfWeek'

    DEVICE = 'Device'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AuctionSegment from a JSON string"""
        return cls(json.loads(json_str))





from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping, _ComplexBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.extensions import *


class BulkCampaignNegativeStoreCriterion(_SingleRecordBulkEntity):
    """ Represents a Campaign Criterion that can be read or written in a bulk file.

    This class exposes the :attr:`negative_campaign_criterion` property that can be read and written as fields of the
    Campaign Negative Store Criterion record in a bulk file.

    For more information, see Campaign Negative Store Criterion at https://go.microsoft.com/fwlink/?linkid=836839.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 campaign_name=None,
                 status=None,
                 negative_campaign_criterion=None):
        super(BulkCampaignNegativeStoreCriterion, self).__init__()

        self._campaign_name = campaign_name
        self._status = status
        self._negative_campaign_criterion = negative_campaign_criterion


    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: c.status,
            csv_to_field=lambda c, v: setattr(c, 'status', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.negative_campaign_criterion.Id),
            csv_to_field=lambda c, v: setattr(c.negative_campaign_criterion, 'Id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.negative_campaign_criterion.CampaignId),
            csv_to_field=lambda c, v: setattr(c.negative_campaign_criterion, 'CampaignId', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Campaign,
            field_to_csv=lambda c: c.campaign_name,
            csv_to_field=lambda c, v: setattr(c, 'campaign_name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.MerchantCenterId,
            field_to_csv=lambda c: bulk_str(c.negative_campaign_criterion.Criterion.StoreId),
            csv_to_field=lambda c, v: setattr(c.negative_campaign_criterion.Criterion,'StoreId', int(v) if v else None)
        ),
    ]

    @property
    def campaign_name(self):
        """ The name of the Campaign

        :rtype: str
        """

        return self._campaign_name

    @campaign_name.setter
    def campaign_name(self, campaign_name):
        self._campaign_name = campaign_name

    @property
    def status(self):
        """ The status of the Campaign Criterion

        :rtype: str
        """

        return self._status

    @status.setter
    def status(self, status):
        self._status = status

    @property
    def negative_campaign_criterion(self):
        """ Defines a Campaign Criterion """

        return self._negative_campaign_criterion

    @negative_campaign_criterion.setter
    def negative_campaign_criterion(self, negative_campaign_criterion):
        self._negative_campaign_criterion = negative_campaign_criterion

    def process_mappings_from_row_values(self, row_values):
        self._negative_campaign_criterion = _CAMPAIGN_OBJECT_FACTORY_V13.create('NegativeCampaignCriterion')
        self._negative_campaign_criterion.Type = 'NegativeCampaignCriterion'
        self._negative_campaign_criterion.Criterion = _CAMPAIGN_OBJECT_FACTORY_V13.create('StoreCriterion')
        self._negative_campaign_criterion.Criterion.Type = 'StoreCriterion'

        row_values.convert_to_entity(self, BulkCampaignNegativeStoreCriterion._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.negative_campaign_criterion, 'negative_campaign_criterion')
        self.convert_to_values(row_values, BulkCampaignNegativeStoreCriterion._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkCampaignNegativeStoreCriterion, self).read_additional_data(stream_reader)
